package com.huoran.iasf.common.aop.aspect;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.Constant;
import com.huoran.iasf.common.utils.R;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import lombok.Synchronized;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import java.util.concurrent.TimeUnit;

import static com.huoran.iasf.service.HttpApiSessionService.APP_SECRET;


/**
 * @Description aop解析注解
 */

@Aspect
@Component
public class NoRepeatSubmitAop {

    private Log logger = LogFactory.getLog(getClass());

    @Autowired
    private RedisServiceAop redisService;

    @Synchronized
    @Around("execution(* com.huoran.*.controller.*.*(..)) && @annotation(noRepeatSubmit)")
    public Object around(ProceedingJoinPoint pjp, NoRepeatSubmit noRepeatSubmit) throws Throwable {

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        HttpServletResponse response = attributes.getResponse();
        String token = request.getHeader(Constant.ACCESS_TOKEN);
        //如果header中不存在token，则从参数中获取token
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter(Constant.ACCESS_TOKEN);
        }
        if (StringUtils.isEmpty(token)) {
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            response.setCharacterEncoding("utf-8");
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            throw new BusinessException(BaseResponseCode.TOKEN_ERROR);
        }
        //  校验并解析token，如果token过期或者篡改，则会返回null
        Claims claims = checkJWT(token);
        if (null == claims) {
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            response.setCharacterEncoding("utf-8");
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            throw new BusinessException(BaseResponseCode.TOKEN_ERROR);
        }
        String key = token + "-" + request.getServletPath();
        if ( !redisService.haskey(key) ) {// 如果缓存中有这个url视为重复提交
            Object o = pjp.proceed();
            redisService.setCacheObject(key, 0, 1, TimeUnit.SECONDS);
            return o;
        } else {
            redisService.setCacheObject(key, 0, 1, TimeUnit.SECONDS);//点了同样的URL继续限制，直到2次点击中间间隔超过了限制
            return R.fail("请勿重复提交或者操作过于频繁！");
        }

    }

    /**
     * 校验token
     */
    public Claims checkJWT(String token) {

        try {
            final Claims claims = Jwts.parser().setSigningKey(APP_SECRET).
                    parseClaimsJws(token).getBody();
            return claims;
        } catch (Exception e) {
            return null;
        }
    }

}
