package com.huoran.iasf.common.shiro;

import com.alibaba.fastjson.JSON;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.UnauthorizedException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.Constant;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.common.utils.HttpContextUtils;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.Subject;
import org.apache.shiro.web.filter.AccessControlFilter;
import org.apache.shiro.web.util.WebUtils;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;

import static com.huoran.iasf.service.HttpApiSessionService.*;
import static javax.servlet.http.HttpServletResponse.SC_UNAUTHORIZED;


/**
 * shiro过滤器 登录用户权限认证授权
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@Slf4j
public class CustomAccessControlFilter extends AccessControlFilter {

//    @Override
//    protected boolean preHandle(ServletRequest request, ServletResponse response) throws Exception {
//        HttpServletRequest httpRequest = WebUtils.toHttp(request);
//        HttpServletResponse httpResponse = WebUtils.toHttp(response);
//        if (httpRequest.getMethod().equals(RequestMethod.OPTIONS.name())) {
//            httpResponse.setHeader("Access-control-Allow-Origin", httpRequest.getHeader("Origin"));
//            httpResponse.setHeader("Access-Control-Allow-Methods", httpRequest.getMethod());
//            //  httpResponse.setHeader("Access-Control-Allow-Methods", "POST, GET, PATCH, DELETE, PUT");
//            httpResponse.setHeader("Access-Control-Allow-Headers", httpRequest.getHeader("Access-Control-Request-Headers"));
//            httpResponse.setStatus(HttpStatus.OK.value());
//            return false;
//        }
//        return super.preHandle(request, response);
//    }


    @Override
    protected boolean isAccessAllowed(ServletRequest servletRequest, ServletResponse servletResponse, Object o) {
        return false;
    }

    @Override
    protected boolean onAccessDenied(ServletRequest servletRequest, ServletResponse servletResponse) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) servletRequest;
//        try {
        HttpServletResponse response = (HttpServletResponse) servletResponse;
            Subject subject = getSubject(servletRequest, servletResponse);
            System.out.println(subject.isAuthenticated() + "");
            System.out.println(HttpContextUtils.isAjaxRequest(request));
            log.info(request.getMethod());
            log.info(request.getRequestURL().toString());
            //从header中获取token
            String token = request.getHeader(Constant.ACCESS_TOKEN);
            System.out.println("token===============" + token);
            //如果header中不存在token，则从参数中获取token
            if (StringUtils.isEmpty(token)) {
                token = request.getParameter(Constant.ACCESS_TOKEN);
            }
            if (StringUtils.isEmpty(token)) {
                response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
                response.setCharacterEncoding("utf-8");
                response.setContentType(MediaType.APPLICATION_JSON_VALUE);
                request.setAttribute("filterError", new UnauthorizedException(BaseResponseCode.TOKEN_ERROR));
                // 指定处理该请求的处理器
                request.getRequestDispatcher(Constant.ERROR_CONTROLLER_PATH).forward(request, response);
            }
            //  校验并解析token，如果token过期或者篡改，则会返回null
            Claims claims = checkJWT(token);
            if (null == claims) {
                response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
                response.setCharacterEncoding("utf-8");
                response.setContentType(MediaType.APPLICATION_JSON_VALUE);
                request.setAttribute("filterError", new UnauthorizedException(BaseResponseCode.TOKEN_ERROR));
                // 指定处理该请求的处理器
                request.getRequestDispatcher(Constant.ERROR_CONTROLLER_PATH).forward(request, response);
            }else {
                Subject subject1 = getSubject(servletRequest, servletResponse);
                UsernamePasswordToken usernamePasswordToken = new UsernamePasswordToken(token, token);
                subject1.login(usernamePasswordToken);
            }
        return true;
    }

    /**
     * 校验token
     */
    public Claims checkJWT(String token) {

        try {
            final Claims claims = Jwts.parser().setSigningKey(APP_SECRET).
                    parseClaimsJws(token).getBody();
            return claims;
        } catch (Exception e) {
            return null;
        }
    }

    private void customResponse(int code, String msg, ServletResponse response) {
        try {
            R result = R.getResult(code, msg);

            response.setContentType("application/json; charset=utf-8");
            response.setCharacterEncoding("UTF-8");

            String userJson = JSON.toJSONString(result);
            OutputStream out = response.getOutputStream();
            out.write(userJson.getBytes(StandardCharsets.UTF_8));
            out.flush();
        } catch (IOException e) {
            log.error("error={}", e, e);
        }
    }

}
