package com.huoran.iasf.common.utils;

import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.vo.req.ExcelImpUserReq;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ExcelImportHelper {

    //判断row是否为空
    public static boolean isRowEmpty(Row row) {
        if (null == row) {
            return true;
        }
        int firstCellNum = row.getFirstCellNum();   //第一个列位置
        int lastCellNum = row.getLastCellNum();     //最后一列位置
        int nullCellNum = 0;    //空列数量
        for (int c = firstCellNum; c < lastCellNum; c++) {
            Cell cell = row.getCell(c);
            if (null == cell || CellType.BLANK == cell.getCellType()) {
                nullCellNum++;
                continue;
            }
            cell.setCellType(CellType.STRING);
            String cellValue = cell.getStringCellValue().trim();
            if (StringUtils.isEmpty(cellValue)) {
                nullCellNum++;
            }
        }
        //所有列都为空
        if (nullCellNum == (lastCellNum - firstCellNum)) {
            return true;
        }
        return false;
    }


    private static Workbook getWorkbook(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        Workbook workbook = null;

        if (fileName.endsWith("xlsx")) {
            try {
                workbook = new XSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else if (fileName.endsWith("xls")) {
            try {
                workbook = new HSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return workbook;
    }


    /**
     * @Description :   读取文件数据
     * @Param file
     * @Author Rong---2021/10/15
     */
    public static List<ExcelImpUserReq> readUser(MultipartFile file) {
        List<ExcelImpUserReq> list = new ArrayList<>();

        Workbook workbook = getWorkbook(file);
        ExcelImpUserReq excelImpUserReq = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        throw new BusinessException(BaseResponseCode.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                Cell realName;//姓名
                Cell userName;//账号
                Cell jobNumber;//工号
                Cell dept;//所在部门
                Cell roleName;//角色
                Cell phone;//手机号
                Cell email;//邮箱
                Cell groupName;//用户组
                if (row != null) {

                    excelImpUserReq = new ExcelImpUserReq();

                    if (row.getCell(7) != null) {
                        row.getCell(7).setCellType(CellType.STRING);
                        groupName = row.getCell(7);
                    } else {
                        groupName = row.createCell(7);
                    }

                    if (row.getCell(6) != null) {
                        row.getCell(6).setCellType(CellType.STRING);
                        email = row.getCell(6);
                    } else {
                        email = row.createCell(6);
                    }

                    if (row.getCell(5) != null) {
                        row.getCell(5).setCellType(CellType.STRING);
                        phone = row.getCell(5);
                    } else {
                        phone = row.createCell(5);
                    }

                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        dept = row.getCell(4);
                    } else {
                        dept = row.createCell(4);
                    }

                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        jobNumber = row.getCell(3);
                    } else {
                        jobNumber = row.createCell(3);
                    }

                    if (row.getCell(2) != null) {
                        row.getCell(2).setCellType(CellType.STRING);
                        roleName = row.getCell(2);
                    } else {
                        roleName = row.createCell(3);
                    }

                    if (row.getCell(1) != null) {
                        row.getCell(1).setCellType(CellType.STRING);
                        userName = row.getCell(1);
                    } else {
                        userName = row.createCell(1);
                    }

                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        realName = row.getCell(0);
                    } else {
                        realName = row.createCell(0);
                    }

                    excelImpUserReq.setRealName(realName.getStringCellValue());
                    excelImpUserReq.setUserName(userName.getStringCellValue());
                    excelImpUserReq.setJobNumber(jobNumber.getStringCellValue());
                    excelImpUserReq.setDept(dept.getStringCellValue());
                    excelImpUserReq.setRoleName(roleName.getStringCellValue());
                    excelImpUserReq.setPhone(phone.getStringCellValue());
                    excelImpUserReq.setEmail(email.getStringCellValue());
                    excelImpUserReq.setGroupName(groupName.getStringCellValue());

                    list.add(excelImpUserReq);
                }
            }

        }

        return list;
    }

}
