package com.huoran.iasf.common.utils;

import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.exception.code.ResponseCodeInterface;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;

/**
 * 返回值R
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@Data
public class R implements Serializable {
    // 类的内容
    private static final long serialVersionUID = 1L;

    /**
     * 请求响应code，0为成功 其他为失败
     */
    @ApiModelProperty(value = "请求响应code，200为成功 其他为失败", name = "code")
    private int code;

    /**
     * 响应异常码详细信息
     */
    @ApiModelProperty(value = "响应异常码详细信息", name = "msg")
    private String msg;

    @ApiModelProperty(value = "需要返回的数据", name = "data")
    private Object data;

    public R(int code, Object data) {
        this.code = code;
        this.data = data;
        this.msg = null;
    }

    public R(int code, String msg, Object data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }

    public R(int code, String msg) {
        this.code = code;
        this.msg = msg;
        this.data = null;
    }

    public R() {
        this.code = BaseResponseCode.SUCCESS.getCode();
        this.msg = BaseResponseCode.SUCCESS.getMsg();
        this.data = null;
    }

    public R(Object data) {
        this.data = data;
        this.code = BaseResponseCode.SUCCESS.getCode();
        this.msg = BaseResponseCode.SUCCESS.getMsg();
    }

    public R(ResponseCodeInterface responseCodeInterface) {
        this.data = null;
        this.code = responseCodeInterface.getCode();
        this.msg = responseCodeInterface.getMsg();
    }

    public R(ResponseCodeInterface responseCodeInterface, Object data) {
        this.data = data;
        this.code = responseCodeInterface.getCode();
        this.msg = responseCodeInterface.getMsg();
    }

    /**
     * 操作成功 data为null
     */
    public static R success() {
        return new R();
    }

    /**
     * 操作成功 data 不为null
     */
    public static R success(Object data) {
        return new R(data);
    }

    /**
     * 操作失败 data 不为null
     */
    public static R fail(String msg) {
        return new R(BaseResponseCode.OPERATION_ERROR.getCode(), msg);
    }

    public static R fail(Object data) {
        return new R(data);
    }


    /**
     * 自定义返回  data为null
     */
    public static R getResult(int code, String msg) {
        return new R(code, msg);
    }

    /**
     * 自定义返回 入参一般是异常code枚举 data为空
     */
    public static R getResult(BaseResponseCode responseCode) {
        return new R(responseCode);
    }


}
