package com.huoran.iasf.common.utils;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.huoran.iasf.entity.SysColumn;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

public class TreeStructureUtils {

    // 获取树结构数据
    public static List<SysColumn> getList(List<SysColumn> columns) {
        try {
            List<SysColumn> result = new ArrayList<>();
            for (SysColumn test : columns) {
                if (test.getFatherId() == 0) {
                    result.add(test);
                }
            }
            for (SysColumn test : result) {
                List<SysColumn> childList = getChildren(test.getId(), columns);
                test.setChildren(childList);
            }
            // 将一层一层的树结构数据返回吧！
            return result;
        } catch (Exception e) {
            // 这里可以抛个异常
        }
        return null;
    }


    //把一个List转成树
    public static List<SysColumn> buildTree(List<SysColumn> list, Integer pid) {
        List<SysColumn> tree = new ArrayList<>();
        for (SysColumn node : list) {
            if (Objects.equals(node.getFatherId(), pid)) {
                tree.add(findChild(node, list));
            }
        }
        return tree;
    }

    static SysColumn findChild(SysColumn node, List<SysColumn> list) {
        for (SysColumn n : list) {
            if (Objects.equals(n.getFatherId(), node.getId())) {
                if (node.getChildren() == null) {
                    node.setChildren(new ArrayList<SysColumn>());
                }
                node.getChildren().add(findChild(n, list));
            }
        }
        return node;
    }

    public static List<SysColumn> getChildren(Integer id, List<SysColumn> allDept) {
        //存放子节点
        List<SysColumn> childList = new ArrayList<>();
        //遍历所有栏目，如果父id与传来的id相同，则为传来的id这个栏目的子栏目
        for (SysColumn dept : allDept) {
            Integer parentId = dept.getFatherId();
            if (parentId.equals(id)) {
                childList.add(dept);
            }
        }

        //自调用来判断是否还有子节点
        for (SysColumn dept : childList) {
            dept.setChildren(getChildren(dept.getId(), allDept));
        }

        //如果没有子节点则返回空集合
        if (childList.size() == 0) {
            return new ArrayList<>();
        }
        return childList;
    }





    /**
     * 双重for循环方法转换成树形结构
     * @param treeList
     * @return
     */
    public static List<SysColumn> forMethod(List<SysColumn> treeList) {
        List<SysColumn> rootTree = new ArrayList<>();
        for (SysColumn tree : treeList) {
            // 第一步 筛选出最顶级的父节点
            if (0 == tree.getFatherId()) {
                rootTree.add(tree);
            }
            // 第二步 筛选出该父节点下的所有子节点列表
            for (SysColumn node : treeList) {
                if (node.getFatherId().equals(tree.getId())) {
                    if (CollectionUtils.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(node);
                }
            }
        }
        return rootTree;
    }

}
