package com.huoran.iasf.common.utils;

import com.huoran.iasf.common.exception.BusinessException;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;

import static com.huoran.iasf.common.exception.code.BaseResponseCode.UPLOAD_FAILED;

/**
 * 文件上传校验工具类
 **/
public class fileUploadUtils {


    // 缓存文件魔数值
    public static final HashMap<String, String> mFileTypes = new HashMap<String, String>();

    static {
        mFileTypes.put("FFD8FFE0", "jpg");
        mFileTypes.put("89504E47", "png");
        mFileTypes.put("47494638", "gif");
//        mFileTypes.put("49492A00", "tif");
//        mFileTypes.put("424D", "bmp");
//        mFileTypes.put("38425053", "psd");
//        mFileTypes.put("3C3F786D6C", "xml");
//        mFileTypes.put("68746D6C3E", "html");
        mFileTypes.put("D0CF11E0", "doc");
        mFileTypes.put("D0CF11E0", "xls");//excel2003版本文件
        mFileTypes.put("6D6F6F76", "mov");
        mFileTypes.put("504B0304", "xlsx");//excel2007以上版本文件
//        mFileTypes.put("5374616E64617264204A", "mdb");
        mFileTypes.put("255044462D312E", "pdf");
        mFileTypes.put("504B0304", "docx");
        mFileTypes.put("00000020667479706D70","MP4");
        mFileTypes.put("49443303000000002176","MP3");
//        mFileTypes.put("52617221", "rar");
//        mFileTypes.put("41564920", "avi");
    }


    /**
     * 文件上传校验
     *
     * @param file             上传的文件
     * @param allowedExtension 允许上传的文件后缀集合
     * @throws Exception
     */
    public static final void assertAllowed(MultipartFile file, String[] allowedExtension) throws Exception {

        //通过文件魔数获取文件的原始类型
        String fileExtension = mFileTypes.get(getFileHeader(file));
        //原始类型与允许类型集合进行比较，判断文件是否合法
        if (!isAllowedExtension(fileExtension, allowedExtension)) {
            throw new BusinessException(UPLOAD_FAILED);
        }
    }

    /**
     * 获取文件魔数值
     *
     * @param file
     * @return
     */
    public static String getFileHeader(MultipartFile file) {
        InputStream is = null;
        String value = null;
        try {
            is = file.getInputStream();
            byte[] b = new byte[4];
            is.read(b, 0, b.length);
            value = bytesToHexString(b);
        } catch (Exception e) {
        } finally {
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                }
            }
        }
        return value;
    }

    private static String bytesToHexString(byte[] src) {
        StringBuilder builder = new StringBuilder();
        if (src == null || src.length <= 0) {
            return null;
        }
        String hv;
        for (int i = 0; i < src.length; i++) {
            hv = Integer.toHexString(src[i] & 0xFF).toUpperCase();
            if (hv.length() < 2) {
                builder.append(0);
            }
            builder.append(hv);
        }
        System.out.println("文件魔数值为：" + builder.toString());
        return builder.toString();
    }

    /**
     * 判断MIME类型是否是允许的MIME类型
     *
     * @param extension
     * @param allowedExtension
     * @return
     */
    public static final boolean isAllowedExtension(String extension, String[] allowedExtension) {
        for (String str : allowedExtension) {
            if (str.equalsIgnoreCase(extension)) {
                return true;
            }
        }
        return false;
    }
}
