package com.huoran.iasf.controller;

import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.aop.annotation.LogAnnotation;
import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysDept;
import com.huoran.iasf.service.DeptService;
import com.huoran.iasf.vo.resp.DeptRespNodeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;

/**
 * 部门管理
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@RequestMapping("/sys")
@RestController
@Api(tags = "部门管理")
public class DeptController {
    @Resource
    private DeptService deptService;
    @NoRepeatSubmit
    @Decrypt
    @PostMapping("/dept")
    @ApiOperation(value = "新增组织接口")
    @LogAnnotation(title = "机构管理", action = "新增组织")
    @RequiresPermissions("sys:dept:add")
    public R addDept(@RequestBody @Valid SysDept vo) throws UnsupportedEncodingException {
        //中文转码
        String name = URLDecoder.decode(vo.getName(), StandardCharsets.UTF_8.toString());
        vo.setName(name);

        String pidName = URLDecoder.decode(vo.getPidName(), StandardCharsets.UTF_8.toString());
        vo.setPidName(pidName);
        boolean ok = deptService.checkDeptRepeat(vo);
        if (ok) {
            return R.fail("同级下已存在相同的名称:" + vo.getName());
        }
        deptService.addDept(vo);
        return R.success();
    }

    @PostMapping("/deleteDept")
    @ApiOperation(value = "删除组织接口")
    @Decrypt
    @LogAnnotation(title = "机构管理", action = "删除组织")
    @RequiresPermissions("sys:dept:deleted")
    public R deleted(@RequestBody String id) {
        deptService.deleted(id);
        return R.success();
    }

    @PostMapping("/updateDept")
    @Decrypt
    @ApiOperation(value = "更新组织信息接口")
    @LogAnnotation(title = "机构管理", action = "更新组织信息")
    @RequiresPermissions("sys:dept:update")
    public R updateDept(@RequestBody SysDept vo) throws UnsupportedEncodingException {
        if (StringUtils.isEmpty(vo.getId())) {
            return R.fail("id不能为空");
        }
        //中文转码
        String name = URLDecoder.decode(vo.getName(), StandardCharsets.UTF_8.toString());
        vo.setName(name);

        String pidName = URLDecoder.decode(vo.getPidName(), StandardCharsets.UTF_8.toString());
        vo.setPidName(pidName);
        boolean ok = deptService.checkDeptRepeat(vo);
        if (ok) {
            return R.fail("同级下已存在相同的名称:" + vo.getName());
        }
        deptService.updateDept(vo);
        return R.success();
    }

    @GetMapping("/dept/{id}")
    @ApiOperation(value = "查询组织详情接口")
    @LogAnnotation(title = "机构管理", action = "查询组织详情")
//    @RequiresPermissions("sys:dept:detail")
    public R detailInfo(@PathVariable("id") String id) {
        return R.success(deptService.getById(id));
    }

    @GetMapping("/dept/tree")
    @ApiOperation(value = "树型组织列表接口",response = DeptRespNodeVO.class)
    @LogAnnotation(title = "部门管理", action = "树型组织列表")
    @RequiresPermissions(value = {"sys:user:list", "sys:user:update", "sys:user:add", "sys:dept:add", "sys:dept:update"}, logical = Logical.OR)
    public R getTree(
            @ApiParam(name = "deptId", value = "部门id",required = false)
            @RequestParam(required = false) String deptId) {
        return R.success(deptService.deptTreeList(deptId));
    }

/*    @GetMapping("/depts")
    @ApiOperation(value = "获取机构列表接口")
    @LogAnnotation(title = "机构管理", action = "获取所有组织机构")
    @RequiresPermissions("sys:dept:list")
    public R getDeptAll() {
        List<SysDept> deptList = deptService.list();
        deptList.parallelStream().forEach(entity -> {
            SysDept parentDept = deptService.getById(entity.getPid());
            if (parentDept != null) {
                entity.setPidName(parentDept.getName());
            }
        });
        return R.success(deptList);
    }*/

}
