package com.huoran.iasf.controller;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huoran.iasf.common.aop.annotation.LogAnnotation;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.Site;
import com.huoran.iasf.service.SiteService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 站点管理 前端控制器
 * </p>
 *
 * @author cheney
 * @since 2022-08-24
 */
@RestController
@RequestMapping("/sys/site")
@Api(tags = "站点管理")
public class SiteController {

    @Autowired
    private SiteService siteService;

    @PostMapping("/add")
//    @ApiOperation(value = "新增站点")
//    @LogAnnotation(title = "站点管理", action = "新增站点")
//    @RequiresPermissions("sys:site:add")
    public R addUserGroup(@RequestBody Site site) {
        Site one = siteService.getOne(new QueryWrapper<Site>().
                eq("site_name", site.getSiteName()));
        if (ObjectUtil.isNotNull(one)){
            R.fail("站点已存在");
        }
        siteService.save(site);
        return R.success();
    }

    @DeleteMapping("/delete/{id}")
//    @ApiOperation(value = "删除站点")
//    @LogAnnotation(title = "站点管理", action = "删除站点")
//    @RequiresPermissions("sys:site:deleted")
    public R deleted(@PathVariable("id") Integer id) {
        siteService.removeById(id);
        return R.success();
    }

    @PutMapping("/update")
//    @ApiOperation(value = "更新站点")
//    @LogAnnotation(title = "站点管理", action = "更新站点")
//    @RequiresPermissions("sys:site:update")
    public R updateUserGroup(@RequestBody Site site) {
        if (StringUtils.isEmpty(site.getId())) {
            return R.fail("id不能为空");
        }
        QueryWrapper<Site> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("domain_name", site.getDomainName());
        queryWrapper.last(" and id  != " + site.getId());
        List<Site> groups = siteService.list(queryWrapper);
        if (ObjectUtil.isNotNull(groups) && groups.size()>0){
            R.fail("域名已存在");
        }
        siteService.updateById(site);
        return R.success();
    }


    @PostMapping("/list")
    @ApiOperation(value = "站点列表")
    @LogAnnotation(title = "站点管理", action = "获取所有站点")
//    @RequiresPermissions("sys:site:list")
    public R getUserGroupAll(@RequestBody Site site) {
        LambdaQueryWrapper<Site> queryWrapper = Wrappers.lambdaQuery();
        if (!StringUtils.isEmpty(site.getSiteName())) {
            queryWrapper.like(Site::getSiteName, site.getSiteName());
        }
        IPage<Site> iPage = siteService.page(site.getQueryPage(),queryWrapper);
        return R.success(iPage);
    }
}

