package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.exception.NotFoundException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.Constant;
import com.huoran.iasf.entity.*;
import com.huoran.iasf.service.*;
import com.huoran.iasf.vo.req.ColumnWeightReqVO;
import com.huoran.iasf.vo.req.OneLevelChecksThemAll;
import com.huoran.iasf.vo.req.PaginationColumnReqVO;
import com.huoran.iasf.vo.req.SubLevelColumnsUnderALevel;
import com.huoran.iasf.vo.resp.SortColumnRespVO;
import io.swagger.annotations.*;
import com.huoran.iasf.common.utils.R;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.Api;
import org.springframework.web.bind.annotation.RequestParam;
import io.swagger.annotations.ApiOperation;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * @描述：栏目基础信息控制类
 * @作者: Rong
 * @日期: 2022-08-01
 */
@RestController
@RequestMapping("/sysColumn")
@Api(value = "栏目基础信息:SysColumnController", tags = "R-栏目管理")
public class SysColumnController {

    @Autowired
    public SysColumnService service;

    @Autowired
    public SysContentService contentService;

    @Autowired
    public SysTemplateService templateService;

    @Autowired
    public SysTemplateStyleService styleService;

    @Autowired
    public SysColumnLongPageService sysColumnLongPageService;

    @Autowired
    public SysTemplateStyleConfigurationService templateStyleConfigurationService;

    @Autowired
    public SysFilesService sysFilesService;

    @PostMapping("/listWithTree")
    @Decrypt
    @ApiOperation(value = "栏目树结构", response = SysColumn.class)
    public R listWithTree(@RequestBody @Valid PaginationColumnReqVO sysColumn) throws UnsupportedEncodingException {
        if (!StringUtils.isEmpty(sysColumn.getColumnName())) {
            String columnName = URLDecoder.decode(sysColumn.getColumnName(), StandardCharsets.UTF_8.toString());
            sysColumn.setColumnName(columnName);
        }
        return R.success(service.listWithTree(sysColumn));
    }

    @PostMapping("/listWithTreeMenuVisible")
    @Decrypt
    @ApiOperation(value = "栏目树结构(前台可见，只展示试单可见的栏目)", response = SysColumn.class)
    public R listWithTreeMenuVisible(@RequestBody @Valid PaginationColumnReqVO sysColumn) {
        return R.success(service.listWithTreeMenuVisible(sysColumn));
    }


    @PostMapping("/findById")
    @Decrypt
    @ApiOperation(value = "查询详情", response = SysColumn.class)
    public R findById(@RequestBody @ApiParam(value = "序号") Integer id) {
        SysColumn sysColumn = service.getById(id);
        if (sysColumn == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }
        SysTemplateStyle sysTemplateStyle = styleService.getById(sysColumn.getListStyleId());

        if (sysTemplateStyle.getPath() != null) {
            sysColumn.setPath(sysTemplateStyle.getPath());
        }

        return R.success(sysColumn);
    }

//    @NoRepeatSubmit
    @PostMapping("/save")
    @ApiOperation(value = "新增", response = SysColumn.class)
    @RequiresPermissions("sys:column:save")
    public R save(@RequestBody @Valid SysColumn sysColumn) {
        QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<SysColumn>();
        queryWrapper.eq("father_id", sysColumn.getFatherId());
        Integer count = service.count(queryWrapper);
        sysColumn.setSort(count + 1);
        boolean addState = service.save(sysColumn);
        return addState ? R.success(sysColumn.getId()) : R.fail("新增失败");
    }

    @PostMapping("/update")
    @ApiOperation(value = "修改", response = SysColumn.class)
    @RequiresPermissions("sys:column:update")
    public R update(@RequestBody SysColumn sysColumn) {
        boolean updateState = service.updateById(sysColumn);
        service.removeArticles(sysColumn.getId());
        return updateState ? R.success() : R.fail("编辑失败");
    }

    /*@NoRepeatSubmit
    @PostMapping("/delete")
    @ApiOperation(value = "删除", response = SysContent.class)
    public R delete(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        UpdateWrapper<SysFilesEntity> filesEntityUpdateWrapper = new UpdateWrapper<>();
        filesEntityUpdateWrapper.eq("quote_id", id);
        filesEntityUpdateWrapper.eq("quote_type", 1);
        filesEntityUpdateWrapper.set("is_del", 1);
        sysFilesService.update(filesEntityUpdateWrapper);

        boolean delState = service.removeById(id);

        QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("column_id", id);
        //删除栏目后删除栏目下的文章
        List<SysContent> contentId = contentService.list(queryWrapper);
        contentService.removeByIds(contentId);

        return delState ? R.success() : R.fail("删除失败");
    }*/

    @NoRepeatSubmit
    @Decrypt
    @PostMapping("/batchDeletion")
    @ApiOperation(value = "批量删除栏目", response = SysColumn.class)
    @RequiresPermissions("sys:column:batchDeletion")
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestBody List<Integer> ids) {
        boolean ret = false;
        for (Integer id : ids) {
            UpdateWrapper<SysFilesEntity> filesEntityUpdateWrapper = new UpdateWrapper<>();
            filesEntityUpdateWrapper.eq("quote_id", id);
            filesEntityUpdateWrapper.eq("quote_type", 1);
            filesEntityUpdateWrapper.set("is_del", 1);
            sysFilesService.update(filesEntityUpdateWrapper);
            //1 创建list集合，用于封装所有删除菜单id值
            List<Integer> idList = new ArrayList<>();
            //2 向idList集合设置删除菜单id
            //找到当前菜单的子菜单  把结果id封装到idlist里面去
            this.selectCategoryChildById(id + "", idList);
            //把当前id封装到list里面 //现在把当前id封装进去  之前都是子菜单的id
            idList.add(id);

            //删除当前栏目以及栏目下的文章
            QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("column_id", idList);
            //删除栏目后删除栏目下的文章
            contentService.remove(queryWrapper);


            //删除长页栏目
            QueryWrapper<SysColumnLongPage> queryWrapper1 = new QueryWrapper<SysColumnLongPage>();
            queryWrapper1.in("column_id", idList);
            //删除栏目后删除栏目下的文章
            sysColumnLongPageService.remove(queryWrapper1);
            ret = service.removeByIds(idList);
            for (Integer columnId : idList) {
                service.removeArticles(columnId);
            }
        }


        return R.success();
    }

    @PostMapping("/sameLevelJudgment")
    @Decrypt
    @ApiOperation(value = "同级判重", response = ColumnWeightReqVO.class)
    public R sameLevelJudgment(@RequestBody @Valid ColumnWeightReqVO sysColumn) throws UnsupportedEncodingException {
        if (!StringUtils.isEmpty(sysColumn.getColumnName())) {
            //中文转码
            String name = URLDecoder.decode(sysColumn.getColumnName(), StandardCharsets.UTF_8.toString());
            sysColumn.setColumnName(name);
        }
        return service.sameLevelJudgment(sysColumn) ? R.success() : R.fail("同级下已存在重复栏目!");
    }

    @PostMapping("/sortByColumn")
    @ApiOperation(value = "栏目排序(依据id更改)", response = SysColumn.class)
    @RequiresPermissions("sys:column:sort")
    public R sortByColumn(@RequestBody List<SortColumnRespVO> sortColumnRespVOList) {
        for (SortColumnRespVO sort : sortColumnRespVOList) {

            SysColumn column = service.getById(sort.getId());

            QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("column_name", column.getColumnName());
            queryWrapper.eq("site_id", column.getSiteId());
            queryWrapper.eq("father_id", sort.getFatherId());
            queryWrapper.last(" and id <> " + sort.getId());
            List<SysColumn> columnList = service.list(queryWrapper);
            if (columnList.size() > 0) {
                return R.fail("同级下已存在重复栏目:" + column.getColumnName());
            }

            SysColumn sysColumn = new SysColumn();
            BeanUtils.copyProperties(sort, sysColumn);
            service.updateById(sysColumn);
        }

        return R.success();
    }

    @ApiOperation("获取一级下的子级栏目")
    @PostMapping("/getsTheSubColumn")
    public R getsTheSubColumn(@ApiParam(name = "id", value = "id", required = true) @RequestParam Integer id,
                              @ApiParam(name = "siteId", value = "站点id", required = true) @RequestParam Integer siteId) {
        String ids = this.getIds(id);
        return service.getsTheSubColumn(id, ids,siteId);

    }


    @ApiOperation("获取某层级下的子级栏目")
    @Decrypt
    @PostMapping("/getsSublevelColumnsUnderALevel")
    public R getsSublevelColumnsUnderALevel(
            @RequestBody SubLevelColumnsUnderALevel subLevelColumnsUnderALevel) {
        String ids = this.getIds(subLevelColumnsUnderALevel.getId());
        return service.getsTheSubColumn(subLevelColumnsUnderALevel.getId(), ids,subLevelColumnsUnderALevel.getSiteId());

    }

    @Cacheable(value = "articles", key = "#id")
    public String getIds(Integer id) {
        //1 创建list集合，用于封装所有删除菜单id值
        List<Integer> idList = new ArrayList<>();
        //2 向idList集合设置删除菜单id
        this.selectCategoryChildById(id + "", idList);//找到当前菜单的子菜单  把结果id封装到idlist里面去
        //把当前id封装到list里面
        idList.add(id);//现在把当前id封装进去  之前都是子菜单的id
        String str = "";
        for (Integer idstr : idList) {
            str += idstr + ",";
        }
        String ids = str.substring(0, str.length() - 1);
        return ids;
    }

    public void selectCategoryChildById(String id, List<Integer> idList) {
        //查询菜单里面子菜单id
        QueryWrapper<SysColumn> wrapper = new QueryWrapper<SysColumn>();
        wrapper.eq("father_id", id);
        wrapper.eq("deleted", Constant.DATA_NOT_DELETED);
        //查询指定列的值
        wrapper.select("id");
        //当前菜单的所有子菜单
        List<SysColumn> childIdList = service.list(wrapper);
        //把childIdList里面菜单id值获取出来，封装idList里面，做递归查询
        //遍历集合得到每一个对象item
        childIdList.stream().forEach(item -> {
            //封装idList里面
            idList.add(item.getId());
            //递归查询
            this.selectCategoryChildById(item.getId() + "", idList);
        });
    }


    @ApiOperation("获取一级下的子级栏目以及下的全部文章标签")
    @Decrypt
    @PostMapping("/getTheFullArticleByColumn")
    public R getTheFullArticleByColumn(
            @ApiParam(name = "id", value = "一级栏目id", required = true)
            @RequestBody Integer id) {
        String ids = this.getIds(id);
        return contentService.getTheFullArticleByColumn(ids);
    }


    @ApiOperation("获取子级栏目id")
    @PostMapping("/getsTheChildColumnId")
    public R getsTheSublevelColumnIdOfTheNextLevel(@ApiParam(name = "id", value = "一级栏目id", required = true) @RequestParam Integer id) {
        //1 创建list集合，用于封装所有删除菜单id值
        List<Integer> idList = new ArrayList<>();
        //2 向idList集合设置删除菜单id
        this.selectCategoryChildById(id + "", idList);//找到当前菜单的子菜单  把结果id封装到idlist里面去
        //把当前id封装到list里面
        idList.add(id);//现在把当前id封装进去  之前都是子菜单的id
        return R.success(idList);

    }

    @ApiOperation("某一级查全部")
    @Decrypt
    @PostMapping("/oneLevelChecksThemAll")
    public R oneLevelChecksThemAll(
            @RequestBody OneLevelChecksThemAll oneLevelChecksThemAll) {
        Integer pid = service.oneLevelChecksThemAll(oneLevelChecksThemAll.getId());
        String ids = this.getIds(pid);
        return service.getSubColumn(pid, ids, oneLevelChecksThemAll.getIsSort(),oneLevelChecksThemAll.getSiteId());
    }


    /**
     * 查询栏目下的文章(若选中栏目与子级全部栏目类型一致返回全部文章，反之查询选中栏目下的文章)
     * 2023.02.22依需求更改为以发布时间排序(非创建时间)
     *
     * @param columnId
     * @return
     */
    @ApiOperation("查询栏目下的文章(若选中栏目与子级全部栏目类型一致返回全部文章，反之查询选中栏目下的文章)")
    @Decrypt
    @PostMapping("/queryArticlesByColumnType")
    public R queryArticlesByColumnType(
            @ApiParam(name = "columnId", value = "栏目id", required = true)
            @RequestBody Integer columnId) {
        String ids = this.getIds(columnId);
        return service.queryArticlesByColumnType(ids, columnId);
    }


    @NoRepeatSubmit
    @PostMapping("/deleteUselessData")
    @ApiOperation(value = "删除无用数据", response = SysColumn.class)
    public R deleteUselessData() {

        QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("deleted", 0).select("id");
        List<SysColumn> list = service.list(queryWrapper);

        for (SysColumn column : list) {
            UpdateWrapper<SysFilesEntity> filesEntityUpdateWrapper = new UpdateWrapper<>();
            filesEntityUpdateWrapper.eq("quote_id", column.getId());
            filesEntityUpdateWrapper.eq("quote_type", 1);
            filesEntityUpdateWrapper.set("is_del", 1);
            sysFilesService.update(filesEntityUpdateWrapper);
            //1 创建list集合，用于封装所有删除菜单id值
            List<Integer> idList = new ArrayList<>();
            //2 向idList集合设置删除菜单id
            //找到当前菜单的子菜单  把结果id封装到idlist里面去
            this.selectCategoryChildById(column.getId() + "", idList);
            //把当前id封装到list里面 //现在把当前id封装进去  之前都是子菜单的id
            idList.add(column.getId());

            //删除当前栏目以及栏目下的文章
            QueryWrapper<SysContent> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.in("column_id", idList);
            //删除栏目后删除栏目下的文章
            contentService.remove(queryWrapper1);


            //删除长页栏目
            QueryWrapper<SysColumnLongPage> queryWrapper2 = new QueryWrapper<SysColumnLongPage>();
            queryWrapper2.in("column_id", idList);
            //删除栏目后删除栏目下的文章
            sysColumnLongPageService.remove(queryWrapper2);
            service.removeByIds(idList);
        }


        return R.success();
    }
}

