package com.huoran.iasf.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.exception.NotFoundException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.Constant;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.service.SysColumnService;
import com.huoran.iasf.vo.req.SysColumnLongRedisVO;
import io.swagger.annotations.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huoran.iasf.common.utils.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.Api;
import org.springframework.web.bind.annotation.RequestParam;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import com.huoran.iasf.service.SysColumnLongPageService;
import com.huoran.iasf.entity.SysColumnLongPage;

import javax.validation.Valid;
import java.util.List;
import java.util.concurrent.TimeUnit;


/**
 * @描述：长页栏目拓展表控制类
 * @作者: Rong
 * @日期: 2022-08-09
 */
@RestController
@RequestMapping("/sysColumnLongPage")
@Api(value = "长页栏目拓展表:SysColumnLongPageController", tags = "R-长页栏目拓展信息")
public class SysColumnLongPageController {

    @Autowired
    public SysColumnLongPageService service;
    @Autowired
    public SysColumnService columnService;
    @Autowired
    private StringRedisTemplate redisTemplate;


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SysColumnLongPage.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        SysColumnLongPage sysColumnLongPage = service.getById(id);
        if (sysColumnLongPage == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }
        return R.success(sysColumnLongPage);
    }


    @PostMapping("/getLongPageInformation")
    @Decrypt
    @ApiOperation(value = "根据栏目id查询类型长页栏目信息", response = SysColumnLongPage.class)
    public R getLongPageInformation(
            @ApiParam(name = "columnId", value = "栏目id", required = true)
            @RequestBody Integer columnId) {
        QueryWrapper<SysColumnLongPage> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("column_id", columnId);
        List<SysColumnLongPage> longPageList = service.list(queryWrapper);
        return R.success(longPageList);
    }


    @PostMapping("/save")
    @ApiOperation(value = "新增", response = SysColumnLongPage.class)
    public R save(@RequestBody @Valid @ApiParam(name = "长页栏目拓展表对象", value = "传入json格式", required = true) SysColumnLongPage sysColumnLongPage) {

        SysColumn column = columnService.getById(sysColumnLongPage.getColumnId());
        if (sysColumnLongPage.getState() == Constant.ARTICLE_NOT_PUBLISHED) {//发布状态(0:草稿 1已发布)
            //发布状态为草稿箱时候 导航菜单是否启用为禁用状态
            column.setMenuVisible(Constant.THE_NAVIGATION_MENU_IS_NOT_VISIBLE);
        } else {
            column.setMenuVisible(Constant.NAVIGATION_MENU_VISIBLE);
        }

        columnService.updateById(column);
        boolean addState = service.save(sysColumnLongPage);
        return addState ? R.success() : R.fail("新增失败");
    }


    @PostMapping("/update")
    @ApiOperation(value = "修改", response = SysColumnLongPage.class)
    public R update(@RequestBody @ApiParam(name = "长页栏目拓展表对象", value = "传入json格式", required = true) SysColumnLongPage sysColumnLongPage) {
        SysColumn column = columnService.getById(sysColumnLongPage.getColumnId());
        if (sysColumnLongPage.getState() == Constant.ARTICLE_NOT_PUBLISHED) {//发布状态(0:草稿 1已发布)
            //发布状态为草稿箱时候 导航菜单是否启用为禁用状态
            column.setMenuVisible(Constant.THE_NAVIGATION_MENU_IS_NOT_VISIBLE);

        } else {
            column.setMenuVisible(Constant.NAVIGATION_MENU_VISIBLE);
        }

        columnService.updateById(column);
        boolean updateState = service.updateById(sysColumnLongPage);
        return updateState ? R.success() : R.fail("编辑失败");
    }


    @PostMapping("/delete")
    @ApiOperation(value = "删除", response = SysColumnLongPage.class)
    public R delete(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        boolean delState = service.removeById(id);
        return delState ? R.success() : R.fail("删除失败");
    }


    @PostMapping("/saveTheCache")
    @ApiOperation(value = "预览至储存(整个页面存储)")
    public R saveTheCache(@RequestBody @ApiParam(name = "预览至储存对象", required = true) SysColumnLongRedisVO vo) {
        String key = "saveTheCache:columnId:" + vo.getColumnId();
        //保存到缓存中
        redisTemplate.opsForValue().set(key, vo.getJson());
        return R.success();
    }


    @PostMapping("/getRedisCache")
    @Decrypt
    @ApiOperation(value = "获取Redis缓存")
    public R getRedisCache(
            @ApiParam(name = "columnId", value = "栏目id主键", required = true)
            @RequestBody Integer columnId) {
        //保存的key名
        String key = "saveTheCache:columnId:" + columnId;
        //查看缓存是否有数据
        String returnValue = redisTemplate.opsForValue().get(key);

        //使用缓存返回数据
        if (returnValue != null) {
            return R.success(returnValue);
        }


        return R.success();
    }

}

