package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.exception.NotFoundException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysContentClassification;
import com.huoran.iasf.service.SysContentClassificationService;
import com.huoran.iasf.vo.req.AllTheQuery;
import com.huoran.iasf.vo.req.CheckForHeavy;
import io.swagger.annotations.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.Api;
import org.springframework.web.bind.annotation.RequestParam;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.List;


/**
 * @描述：文章所属分类控制类
 * @作者: Rong
 * @日期: 2022-11-08
 */
@RestController
@RequestMapping("/content/classification")
@Api(value = "文章所属分类:SysContentClassificationController", tags = "R-文章所属分类")
public class SysContentClassificationController {

    @Autowired
    public SysContentClassificationService service;


    @PostMapping("/allTheQuery")
    @Decrypt
    @ApiOperation(value = "查询全部文章所属分类", response = SysContentClassification.class)
    public R allTheQuery(
            @RequestBody AllTheQuery allTheQuery) {
        List<SysContentClassification> list = service.list(new QueryWrapper<SysContentClassification>()
                .eq("site_id", allTheQuery.getSiteId()).eq("template_id",allTheQuery.getTemplateId()));
        return R.success(list);
    }


    @PostMapping("/findById")
    @ApiOperation(value = "查询详情", response = SysContentClassification.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestParam Integer id) {
        SysContentClassification sysContentClassification = service.getById(id);
        if (sysContentClassification == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }
        return R.success(sysContentClassification);
    }


    @PostMapping("/checkForHeavy")
    @Decrypt
    @ApiOperation(value = "分类校验判重", response = SysContentClassification.class)
    public R checkForHeavy(
            @RequestBody CheckForHeavy checkForHeavy) throws UnsupportedEncodingException {
        String name = URLDecoder.decode(checkForHeavy.getClassificationName(), StandardCharsets.UTF_8.toString());
        checkForHeavy.setClassificationName(name);
        QueryWrapper<SysContentClassification> queryWrapper = new QueryWrapper<SysContentClassification>().eq("site_id", checkForHeavy.getSiteId()).
                eq("classification_name", checkForHeavy.getClassificationName());

        //id不得空表示编辑校验
        if (checkForHeavy.getClassificationId() != null) {
            queryWrapper.last(" and id != " + checkForHeavy.getClassificationId());
        }
        SysContentClassification sysContentClassification = service.getOne(queryWrapper);
        if (sysContentClassification != null) {
            return R.fail("当前分类名称已存在!");
        }
        return R.success();

    }


    @PostMapping("/save")
    @Decrypt
    @ApiOperation(value = "新增", response = SysContentClassification.class)
    public R save(@RequestBody @Valid @ApiParam(name = "文章所属分类对象", value = "传入json格式", required = true) SysContentClassification sysContentClassification) throws UnsupportedEncodingException {
        String name = URLDecoder.decode(sysContentClassification.getClassificationName(), StandardCharsets.UTF_8.toString());
        sysContentClassification.setClassificationName(name);

        boolean addState = service.save(sysContentClassification);
        return addState ? R.success() : R.fail("新增失败");
    }


    @PostMapping("/update")
    @Decrypt
    @ApiOperation(value = "修改", response = SysContentClassification.class)
    public R update(@RequestBody @ApiParam(name = "文章所属分类对象", value = "传入json格式", required = true) SysContentClassification sysContentClassification) throws UnsupportedEncodingException {
        String name = URLDecoder.decode(sysContentClassification.getClassificationName(), StandardCharsets.UTF_8.toString());
        sysContentClassification.setClassificationName(name);
        boolean updateState = service.updateById(sysContentClassification);
        return updateState ? R.success() : R.fail("编辑失败");
    }


    @PostMapping("/delete")
    @Decrypt
    @ApiOperation(value = "删除", response = SysContentClassification.class)
    public R delete(@ApiParam(name = "id", value = "主键", required = true) @RequestBody Integer id) {
        boolean delState = service.removeById(id);
        return delState ? R.success() : R.fail("删除失败");
    }

    @PostMapping("/batchDeletion")
    @Decrypt
    @ApiOperation(value = "批量删除", response = SysContentClassification.class)
    public R batchDeletion(@ApiParam(name = "id", value = "主键", required = true) @RequestBody List<Integer> ids) {
        boolean delState = service.removeByIds(ids);
        return delState ? R.success() : R.fail("删除失败");
        //
    }
}

