package com.huoran.iasf.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.exception.NotFoundException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.entity.SysContentFile;
import com.huoran.iasf.entity.SysFilesEntity;
import com.huoran.iasf.service.SysColumnService;
import com.huoran.iasf.service.SysContentFileService;
import com.huoran.iasf.service.SysFilesService;
import com.huoran.iasf.vo.req.*;
import com.huoran.iasf.vo.resp.PageContentRespVO;
import io.swagger.annotations.*;
import com.huoran.iasf.common.utils.R;
import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.annotations.Update;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.Api;
import org.springframework.web.bind.annotation.RequestParam;
import io.swagger.annotations.ApiOperation;
import com.huoran.iasf.service.SysContentService;
import com.huoran.iasf.entity.SysContent;

import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * @描述：文章管理控制类
 * @作者: Rong
 * @日期: 2022-08-05
 */
@RestController
@RequestMapping("/sysContent")
@Api(value = "文章管理:SysContentController", tags = "R-文章管理")
public class SysContentController {

    @Autowired
    public SysContentService service;
    @Autowired
    public SysColumnService columnService;

    @Autowired
    public SysContentFileService fileService;

    @Autowired
    public SysFilesService sysFilesService;


    @PostMapping("/pagingQuery")
    @Decrypt
    @ApiOperation(value = "分页查询文章列表", response = PageContentRespVO.class)
    public R pagingQuery(@RequestBody @Valid @ApiParam(name = "分页查询参数", value = "传入json格式", required = true) PageContentReqVO sysContent) throws UnsupportedEncodingException {
        if (!StringUtils.isEmpty(sysContent.getTitle())) {
            //中文转码
            String title = URLDecoder.decode(sysContent.getTitle(), StandardCharsets.UTF_8.toString());
            sysContent.setTitle(title);

        }
        if (!StringUtils.isEmpty(sysContent.getFounder())) {
            //中文转码
            String founder = URLDecoder.decode(sysContent.getFounder(), StandardCharsets.UTF_8.toString());
            sysContent.setFounder(founder);

        }
        if (!StringUtils.isEmpty(sysContent.getColumn())) {
            //中文转码
            String column = URLDecoder.decode(sysContent.getColumn(), StandardCharsets.UTF_8.toString());
            sysContent.setColumn(column);

        }
        if (!StringUtils.isEmpty(sysContent.getEditor())) {
            //中文转码
            String editor = URLDecoder.decode(sysContent.getEditor(), StandardCharsets.UTF_8.toString());
            sysContent.setEditor(editor);

        }
        return service.articlePaginationList(sysContent);
    }


    @PostMapping("/findById")
    @Decrypt
    @ApiOperation(value = "查询详情", response = SysContent.class)
    public R findById(@ApiParam(name = "id", value = "主键", required = true) @RequestBody Integer id) {


        SysContent sysContent = service.getById(id);
        if (sysContent == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }

        SysColumn sysColumn = columnService.getById(sysContent.getColumnId());
        if (sysColumn == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }
        /*if (sysColumn == null) {
            throw new NotFoundException(BaseResponseCode.DATA_DOES_NOT_EXIST);
        }*/
        if (sysColumn.getColumnName() != null) {
            sysContent.setColumnName(sysColumn.getColumnName());
        }
        List<SysContentFile> fileList = fileService.getFileByContentId(id);
        sysContent.setFileList(fileList);


        return R.success(sysContent);
    }

    @PostMapping("/save")
    @RequiresPermissions("sys:article:save")
    @ApiOperation(value = "新增", response = SysContent.class)
    public R save(
            @ApiParam(name = "文章管理对象", value = "传入json格式", required = true)
            @RequestBody @Valid SysContent sysContent) {

      /*  QueryWrapper<SysContent> queryWrap = new QueryWrapper<>();
        queryWrap.eq("column_id", sysContent.getColumnId());
        queryWrap.eq("deleted", 1);
        Integer count = service.count(queryWrap);
        if (count == 0) {
            count = 1;
        }
        sysContent.setSequence(count + 1);*/
        boolean addState = service.save(sysContent);
        columnService.removeArticles(sysContent.getColumnId());

        return addState ? R.success(sysContent.getId()) : R.fail("新增失败");
    }


    @PostMapping("/update")
    @RequiresPermissions("sys:article:update")
    @ApiOperation(value = "修改", response = SysContent.class)
    public R update(@RequestBody @ApiParam(name = "文章管理对象", value = "传入json格式", required = true) SysContent sysContent) {
        boolean updateState = service.updateById(sysContent);
        columnService.removeArticles(sysContent.getColumnId());
        return updateState ? R.success() : R.fail("编辑失败");
    }


    @PostMapping("/delete")
    @Decrypt
    @RequiresPermissions("sys:article:delete")
    @ApiOperation(value = "删除", response = SysContent.class)
    public R delete(@ApiParam(name = "id", value = "主键", required = true) @RequestBody Integer id) {
        UpdateWrapper<SysFilesEntity> queryWrapper = new UpdateWrapper<>();
        queryWrapper.eq("quote_id", id);
        queryWrapper.eq("quote_type", 0);
        queryWrapper.set("is_del", 1);
        sysFilesService.update(queryWrapper);
        SysContent sysContent = service.getById(id);
        columnService.removeArticles(sysContent.getColumnId());
        boolean delState = service.removeById(id);
        return delState ? R.success() : R.fail("删除失败");
    }


    @PostMapping("/batchDeletion")
    @Decrypt
    @ApiOperation(value = "批量删除文章", response = SysContent.class)
    public R batchDeletion(@ApiParam(name = "ids", value = "主键", required = true) @RequestBody List<Integer> ids) {
        for (Integer id : ids) {
            SysContent sysContent = service.getById(id);
            columnService.removeArticles(sysContent.getColumnId());
            UpdateWrapper<SysFilesEntity> queryWrapper = new UpdateWrapper<>();
            queryWrapper.eq("quote_id", id);
            queryWrapper.eq("quote_type", 0);
            queryWrapper.set("is_del", 1);
            sysFilesService.update(queryWrapper);
        }
        boolean delState = service.removeByIds(ids);
        return delState ? R.success() : R.fail("删除失败");
    }

    @PostMapping("/checkIfTheTitleIsRepeat")
//    @Decrypt
    @ApiOperation(value = "标题判重——只对已发布的判重(新增，编辑，点击发布时候都要调用判断))", response = ContentHeavyTitleReqVO.class)
    public R checkIfTheTitleIsRepeat(@RequestBody @Valid ContentHeavyTitleReqVO content) throws UnsupportedEncodingException {
//        if (!org.springframework.util.StringUtils.isEmpty(content.getTitle())) {
//            //中文转码
//            String name = URLDecoder.decode(content.getTitle(), StandardCharsets.UTF_8.toString());
//            content.setTitle(name);
//        }
        return service.checkIfTheTitleIsRepeat(content) ? R.success() : R.fail("该标题已重复!");
    }

    @PostMapping("/newlyPublishedArticles")
    @Decrypt
    @ApiOperation(value = "站点最新发布的文章", response = PageContentReqVO.class)
    public R newlyPublishedArticles(@Valid @RequestBody PageContentReqVO content) throws UnsupportedEncodingException {
        //中文转码
        if (!StringUtils.isEmpty(content.getTitle())) {
            String title = URLDecoder.decode(content.getTitle(), StandardCharsets.UTF_8.toString());
            content.setTitle(title);
        }
        if (!StringUtils.isEmpty(content.getPatentQueryKeyWord())) {
            String title = URLDecoder.decode(content.getPatentQueryKeyWord(), StandardCharsets.UTF_8.toString());
            content.setPatentQueryKeyWord(title);
        }
        if (!StringUtils.isEmpty(content.getMonographQueryKeyWord())) {
            String title = URLDecoder.decode(content.getMonographQueryKeyWord(), StandardCharsets.UTF_8.toString());
            content.setMonographQueryKeyWord(title);
        }
        if (!StringUtils.isEmpty(content.getPaperQueryKeyWord())) {
            String title = URLDecoder.decode(content.getPaperQueryKeyWord(), StandardCharsets.UTF_8.toString());
            content.setPaperQueryKeyWord(title);
        }
        return service.newlyPublishedArticles(content);
    }


    /*查询已发布的、启用了的文章列表接口*/
/*    @PostMapping("/publishedEnabledArticles")
    @ApiOperation(value = "查询已发布的、启用了的文章列表", response = PageContentReqVO.class)
    public R publishedEnabledArticles() {
        return service.publishedEnabledArticles();
    }*/


    /**
     * 获取栏目下的文章
     *
     * @param pageReq
     * @return
     */
    @PostMapping("/getColumnArticles")
    @ApiOperation(value = "获取栏目下的文章", response = SysContent.class)
    public R getColumnArticles(@RequestBody @ApiParam(name = "文章管理对象", value = "传入json格式", required = true) ContentReq pageReq) {
        IPage page = new Page(pageReq.getPageNum(), pageReq.getPageSize());
        QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("column_id", pageReq.getColumnIds());
        IPage<SysContent> iPage = service.page(page, queryWrapper);
        return R.success(iPage);
    }


    @PostMapping("/articleEnableOrDisable")
    @Decrypt
    @RequiresPermissions("sys:article:enable")
    @ApiOperation(value = "文章启用禁用", response = SysContent.class)
    public R articleEnableOrDisable(
            @RequestBody ArticleEnableOrDisable articleEnableOrDisable) {
        UpdateWrapper<SysContent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("is_disable", articleEnableOrDisable.getIsDisable());
        updateWrapper.eq("id", articleEnableOrDisable.getId());
        boolean ret = service.update(updateWrapper);
        return ret ? R.success() : R.fail("禁用/启用失败");
    }

    @PostMapping("/articlePreview")
    @Decrypt
    @ApiOperation(value = "增加文章浏览量(点击一次算一次)", response = SysContent.class)
    public R articlePreview(
            @ApiParam(name = "contentId", value = "文章id", required = true)
            @RequestBody Integer contentId) {
        return R.success(service.statisticsOfPageViews(contentId));

    }


    @PostMapping("/hotContent")
    @Decrypt
    @ApiOperation(value = "热点内容", response = SysContent.class)
    public R hotContent(
            @ApiParam(name = "siteId", value = "站点id", required = true)
            @RequestBody Integer siteId) {
        return R.success(service.hotContent(siteId));

    }

    @PostMapping("/siteSearchArticles")
    @ApiOperation(value = "站点搜索文章", response = SysContent.class)
    public R siteSearchArticles(@RequestBody PageContentReqVO req) {
        return service.siteSearchArticles(req);

    }


    @PostMapping("/articleTopOperation")
    @Decrypt
    @RequiresPermissions("sys:article:top")
    @ApiOperation(value = "文章置顶/取消置顶", response = SysContent.class)
    public R articleTopOperation(
            @RequestBody ArticleTopOperation articleTopOperation) {
        //是否置顶(默认为0 不置顶 1为置顶)
        UpdateWrapper<SysContent> updateWrap = new UpdateWrapper<>();

        if (articleTopOperation.getIsTop() == 1) {
            //置顶时间(置顶一次更新一次)
            updateWrap.set("top_time", new Date());
        } else {
            updateWrap.set("top_time", null);
        }
        updateWrap.set("is_top", articleTopOperation.getIsTop());
        updateWrap.eq("id", articleTopOperation.getArticleId());
        updateWrap.eq("deleted", 1);
        boolean ret = service.update(updateWrap);
        return ret ? R.success() : R.fail("操作失败");

    }


    @PostMapping("/queryArticleColumnParent")
    @Decrypt
    @ApiOperation(value = "查询文章的栏目父级", response = PageContentReqVO.class)
    public R queryArticleColumnParent(
            @ApiParam(name = "articleId", value = "文章id", required = true)
            @RequestBody Integer articleId) {
        return service.queryArticleColumnParent(articleId);
    }



}

