package com.huoran.iasf.controller;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.config.NonStaticResourceHttpRequestConfig;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.common.utils.fileUploadUtils;
import com.huoran.iasf.entity.SysFilesEntity;
import com.huoran.iasf.entity.SysUser;
import com.huoran.iasf.service.HttpSessionService;
import com.huoran.iasf.service.SysFilesService;
import com.huoran.iasf.service.UserService;
import com.huoran.iasf.vo.UEditorResultVO;
import com.huoran.iasf.vo.req.FileParameters;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.resource.ResourceHttpRequestHandler;

import javax.annotation.Resource;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.regex.Pattern;


/**
 * 文件上传
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@RestController
@RequestMapping("/sysFiles")
@Api(tags = "文件管理")
public class SysFilesController {
    @Resource
    private SysFilesService sysFilesService;

    @Autowired
    private UserService userService;

    @Resource
    private HttpSessionService httpSessionService;

    @ApiOperation(value = "上传文件前获取配置文件")
    @GetMapping("/upload")
    public String upload(@RequestParam(required = false) String action,HttpServletResponse response) throws IOException {
        // response.sendRedirect("static/ueditor/config.json");
        // return "redirect:static/ueditor/config.json";
        // 将获取的json数据封装一层，然后在给返回
        JSONObject result = new JSONObject();
        result.put("imageActionName", "upload");
        result.put("imageFieldName", "file");
        return result.toJSONString();
    }
//    @NoRepeatSubmit
    @ApiOperation(value = "新增")
    @PostMapping("/upload")
//    @RequiresPermissions(value = {"sysFiles:add", "sysContent:update", "sysContent:add"}, logical = Logical.OR)
    public UEditorResultVO add(
            @RequestParam(required = false) String action,
            @RequestParam(value = "file") MultipartFile file, FileParameters fileParameters) throws Exception {
        Integer userId = httpSessionService.getCurrentUserId();
        fileParameters.setUploader(userId.toString());
        //判断文件是否空
        if (file == null || file.getOriginalFilename() == null || "".equalsIgnoreCase(file.getOriginalFilename().trim())) {
            UEditorResultVO uEditorResult = new UEditorResultVO();
            uEditorResult.setState("error");
            return uEditorResult;
        }

        String pattern = ".*\\.(mp4|MP3|MP4|jpg|png|gif|doc|xls|xlsx|docx|pdf|mp3)$";
        boolean hasExtension = Pattern.matches(pattern, file.getOriginalFilename());
        if (!hasExtension){
            String[] allowedExtension = new String[]{"jpg", "png", "gif", "doc", "xls", "xlsx", "docx", "pdf", "mov","mp4","MP3"};
            fileUploadUtils.assertAllowed(file,allowedExtension);
        }
        return sysFilesService.saveFile(file,fileParameters);
    }


    @ApiOperation(value = "更新文件信息")
    @PostMapping("/update")
    public R update(@RequestBody FileParameters fileParameters) {
        if (StrUtil.isEmpty(fileParameters.getId())){
            return R.fail("id不能为空");
        }
        UpdateWrapper<SysFilesEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("quote",fileParameters.getQuote()).
                set("is_release",fileParameters.getIsRelease()).
                set("quote_id",fileParameters.getQuoteId()).
                eq("id", fileParameters.getId());
        return sysFilesService.update(updateWrapper) ? R.success() : R.fail("更新失败");
    }
    @ApiOperation(value = "文件预览")
    @GetMapping("/preview/{id}")
    public void preview(@PathVariable String id, HttpServletRequest req, HttpServletResponse res) throws IOException, ServletException {
        sysFilesService.preview(id, req, res);
    }
//    @NoRepeatSubmit
    @ApiOperation(value = "文件下载")
    @GetMapping("/download/{id}")
    public void download(@PathVariable String id, HttpServletRequest req, HttpServletResponse res) {
        sysFilesService.download(id, req, res);
    }

    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    @Decrypt
    @RequiresPermissions("sysFiles:delete")
    public R delete(@RequestBody @ApiParam(value = "id集合") List<String> ids) {
        sysFilesService.removeByIdsAndFiles(ids);
        return R.success();
    }

    @ApiOperation(value = "查询分页数据",response = SysFilesEntity.class)
    @PostMapping("/listByPage")
    @RequiresPermissions("sysFiles:list")
    public R findListByPage(@RequestBody SysFilesEntity sysFiles) throws UnsupportedEncodingException {
        if (!StringUtils.isEmpty(sysFiles.getQuote())) {
            //中文转码
            String title = URLDecoder.decode(sysFiles.getQuote(), StandardCharsets.UTF_8.toString());
            sysFiles.setQuote(title);
        }
        if (!StringUtils.isEmpty(sysFiles.getFileName())) {
            //中文转码
            String title = URLDecoder.decode(sysFiles.getFileName(), StandardCharsets.UTF_8.toString());
            sysFiles.setFileName(title);
        }
        if (!StringUtils.isEmpty(sysFiles.getUploader())) {
            //中文转码
            String title = URLDecoder.decode(sysFiles.getUploader(), StandardCharsets.UTF_8.toString());
            sysFiles.setUploader(title);
        }
        if (!StringUtils.isEmpty(sysFiles.getType())) {
            //中文转码
            String title = URLDecoder.decode(sysFiles.getType(), StandardCharsets.UTF_8.toString());
            sysFiles.setType(title);
        }
        return sysFilesService.getPage(sysFiles);

        /*LambdaQueryWrapper<SysFilesEntity> queryWrapper = Wrappers.lambdaQuery();
        if (!StringUtils.isEmpty(sysFiles.getFileName())) {
            queryWrapper.like(SysFilesEntity::getFileName, sysFiles.getFileName());
        }
        if (!StringUtils.isEmpty(sysFiles.getUploader())) {
            queryWrapper.like(SysFilesEntity::getUploader, sysFiles.getUploader());
        }
        if (!StringUtils.isEmpty(sysFiles.getQuote())) {
            queryWrapper.like(SysFilesEntity::getQuote, sysFiles.getQuote());
        }
        if (!StringUtils.isEmpty(sysFiles.getType())) {
            queryWrapper.eq(SysFilesEntity::getType, sysFiles.getType());
        }
        if (!StringUtils.isEmpty(sysFiles.getSite())) {
            queryWrapper.eq(SysFilesEntity::getSite, sysFiles.getSite());
        }
        queryWrapper.orderByDesc(SysFilesEntity::getCreateDate);
        queryWrapper.isNotNull(SysFilesEntity::getQuote);
        //只查看已发布的文件
        queryWrapper.eq(SysFilesEntity::getIsRelease,1);
        //查看文章未删除的
        queryWrapper.eq(SysFilesEntity::getIsDel,0);
        IPage<SysFilesEntity> iPage = sysFilesService.page(sysFiles.getQueryPage(),queryWrapper);
        iPage.getRecords().forEach(sysFilesEntity -> {
            if (sysFilesEntity.getUploader()!=null){
                SysUser sysUser = userService.getById(sysFilesEntity.getUploader());
                if (sysUser!=null){
                    sysFilesEntity.setUploader(sysUser.getRealName());
                }
            }
        });
        return R.success(iPage);*/
    }

}
