package com.huoran.iasf.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.aop.annotation.LogAnnotation;
import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.common.utils.fileUploadUtils;
import com.huoran.iasf.entity.SysUser;
import com.huoran.iasf.entity.UserDept;
import com.huoran.iasf.mapper.UserDeptMapper;
import com.huoran.iasf.service.HttpSessionService;
import com.huoran.iasf.service.UserService;
import com.huoran.iasf.vo.CheckVo;
import com.huoran.iasf.vo.req.UpdatePhoneOrEmailReq;
import com.huoran.iasf.vo.req.UserListReq;
import com.huoran.iasf.vo.resp.UserListResp;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.apache.shiro.subject.Subject;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.naming.Name;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * 用户管理
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@RestController
@Api(tags = "用户管理")
@RequestMapping("/sys")
@Slf4j
public class UserController {
    @Resource
    private UserService userService;

    @Resource
    private UserDeptMapper userDeptMapper;

    @Resource
    private HttpSessionService httpSessionService;

    @PostMapping(value = "/user/login")
    @Decrypt
    @ApiOperation(value = "用户登录接口")
    public R login(@RequestBody @Valid SysUser vo) {
        return R.success(userService.login(vo));
    }

/*    @PostMapping("/user/register")
    @ApiOperation(value = "用户注册接口")
    public R register(@RequestBody @Valid SysUser vo) {
        userService.register(vo);
        return R.success();
    }*/

    @PostMapping("/updateUser")
    @ApiOperation(value = "更新用户信息接口")
    @LogAnnotation(title = "用户管理", action = "更新用户信息")
    @Decrypt
    @RequiresPermissions("sys:user:update")
    public R updateUserInfo(@RequestBody SysUser vo) throws UnsupportedEncodingException {
        //中文转码
        if (!StringUtils.isEmpty(vo.getRealName())) {
            String realName = URLDecoder.decode(vo.getRealName(), StandardCharsets.UTF_8.toString());
            vo.setRealName(realName);
        }
        if (!StringUtils.isEmpty(vo.getUsername())) {
            String realName = URLDecoder.decode(vo.getUsername(), StandardCharsets.UTF_8.toString());
            vo.setUsername(realName);
        }

        if (StringUtils.isEmpty(vo.getId())) {
            return R.fail("id不能为空");
        }
        userService.updateUserInfo(vo);
        return R.success();
    }

    @PostMapping("/queryUser")
    @ApiOperation(value = "查询用户详情接口")
    @Decrypt
    @LogAnnotation(title = "用户管理", action = "查询用户详情")
    @RequiresPermissions("sys:user:detail")
    public R detailInfo(@RequestBody Integer id) {
        UserListResp userResp = userService.detailInfo(id);
        return R.success(userResp);
    }

    @PostMapping("/personalData")
    @ApiOperation(value = "查询个人资料接口")
    @LogAnnotation(title = "用户管理", action = "查询个人资料接口")
    public R personalData() {
        Integer currentUserId = httpSessionService.getCurrentUserId();
        UserListResp userResp = userService.detailInfo(currentUserId);
        return R.success(userResp);
    }

    @PostMapping("/users")
    @ApiOperation(value = "分页获取用户列表接口")
    @RequiresPermissions("sys:user:list")
    @Decrypt
    @LogAnnotation(title = "用户管理", action = "分页获取用户列表")
    public R pageInfo(@RequestBody UserListReq userListReq) throws UnsupportedEncodingException {
        //中文转码
        String key = URLDecoder.decode(userListReq.getKeyWord(), StandardCharsets.UTF_8.toString());
        userListReq.setKeyWord(key);
        return userService.pageInfo(userListReq);
    }

    @PostMapping("/user")
    @ApiOperation(value = "新增用户接口")
    @RequiresPermissions("sys:user:add")
    @Decrypt
    @LogAnnotation(title = "用户管理", action = "新增用户")
    public R addUser(@RequestBody @Valid SysUser vo) throws UnsupportedEncodingException {
        //中文转码
        if (!StringUtils.isEmpty(vo.getRealName())) {
            String realName = URLDecoder.decode(vo.getRealName(), StandardCharsets.UTF_8.toString());
            vo.setRealName(realName);
        }
        if (!StringUtils.isEmpty(vo.getUsername())) {
            String realName = URLDecoder.decode(vo.getUsername(), StandardCharsets.UTF_8.toString());
            vo.setUsername(realName);
        }
        userService.addUser(vo);
        return R.success();
    }

    @GetMapping("/checkJobNumber")
    @ApiOperation(value = "工号验证")
    public R checkJobNumber(@RequestParam String jobNumber) {
        SysUser sysUser = userService.getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getJobNumber, jobNumber));
        if (sysUser != null) {
            return R.fail("工号已存在，请勿重复添加！");
        }
        return R.success();
    }

    @GetMapping("/checkUsername")
    @ApiOperation(value = "账号验证")
    public R checkUsername(@RequestParam String userName) {
        SysUser sysUser = userService.getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, userName));
        if (sysUser != null) {
            return R.fail("账号已存在，请勿重复添加！");
        }
        return R.success();
    }

    @GetMapping("/checkEmail")
    @ApiOperation(value = "邮箱验证")
    public R checkEmail(@RequestParam String email) {
        SysUser sysUser = userService.getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getEmail, email));
        if (sysUser != null) {
            return R.fail("邮箱已存在，请勿重复添加！");
        }
        return R.success();
    }

    @GetMapping("/checkPhone")
    @ApiOperation(value = "手机验证")
    public R checkPhone(@RequestParam String phone) {
        SysUser sysUser = userService.getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getPhone, phone));
        if (sysUser != null) {
            return R.fail("手机号已存在，请勿重复添加！");
        }
        return R.success();
    }
    @NoRepeatSubmit
    @ApiOperation(value = "密码重置", notes = "密码重置")
    @Decrypt
    @PostMapping("/resetPwd")
    @RequiresPermissions("sys:user:resetPwd")
    public R resetPwd(@RequestBody String userId) {
//        Integer currentUserId = httpSessionService.getCurrentUserId();
//        if (!userId.equals(currentUserId.toString())){
//            return R.fail("只能修改自己的密码");
//        }
        return userService.resetPwd(userId);
    }

    @GetMapping("/user/logout")
    @ApiOperation(value = "退出接口")
    @LogAnnotation(title = "用户管理", action = "退出")
    public R logout() {
        httpSessionService.abortUserByToken();
        Subject subject = SecurityUtils.getSubject();
        subject.logout();
        return R.success();
    }
    @NoRepeatSubmit
    @Decrypt
    @PostMapping("/user/pwd")
    @ApiOperation(value = "修改密码接口")
    @LogAnnotation(title = "用户管理", action = "更新密码")
    public R updatePwd(@RequestBody SysUser vo) {
        if (StringUtils.isEmpty(vo.getOldPwd()) || StringUtils.isEmpty(vo.getNewPwd())) {
            return R.fail("旧密码与新密码不能为空");
        }
         Integer userId = httpSessionService.getCurrentUserId();
         if (!userId.equals(vo.getId())){
             return R.fail("只能修改自己的密码");
         }
        boolean password = userService.checkPassword(vo.getNewPwd());
         if (!password){
             return R.fail("密码长度不小于8位,至少包含数字、大小写字母和特殊字符");
         }
        userService.updatePwd(vo);
        return R.success();
    }
    @NoRepeatSubmit
    @PostMapping("/deletedUser")
    @ApiOperation(value = "删除用户接口")
    @Decrypt
    @LogAnnotation(title = "用户管理", action = "删除用户")
    @RequiresPermissions("sys:user:deleted")
    public R deletedUser(@RequestBody @ApiParam(value = "用户id集合") List<String> userIds) {
        //删除用户， 删除redis的绑定的角色跟权限
        httpSessionService.abortUserByUserIds(userIds);
        LambdaQueryWrapper<SysUser> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(SysUser::getId, userIds);
        userService.remove(queryWrapper);
        //删除用户部门
        userDeptMapper.delete(Wrappers.<UserDept>lambdaQuery().in(UserDept::getUserId, userIds));
        return R.success();
    }

    @PostMapping("/importStaff")
    @ApiOperation("批量导入员工")
    @RequiresPermissions("sys:user:import")
    public R importStaff(@RequestParam(name = "file") MultipartFile file, HttpServletRequest request) throws IOException {
        Map<String, String> map = userService.importStaff(file);
        return R.success(map);
    }

    /**
     * 批量导入员工失败数据导出
     */
    @ApiOperation(value = "批量导入员工失败数据导出")
    @GetMapping("/exportFailure")
    public void exportFailureRecord(HttpServletResponse response, @ApiParam(name = "exportCode", value = "importQuestion接口返回的exportCode", required = true) @RequestParam String exportCode) throws Exception {
        userService.exportFailureRecord(response, exportCode);
    }
    @NoRepeatSubmit
    @ApiOperation(value = "更换手机号或邮箱--发送验证码")
    @PostMapping("/sendPhoneOrEmailCode")
    public R sendPhoneOrEmailCode(@RequestBody CheckVo checkVo) {
        return userService.sendCheckPhoneCode(checkVo);
    }
    @NoRepeatSubmit
    @ApiOperation(value = "更新手机号或邮箱")
    @PostMapping("/updatePhoneOrEmail")
    public R updatePhoneOrEmail(@RequestBody UpdatePhoneOrEmailReq updatePhoneOrEmailReq) {
        return userService.updatePhoneOrEmail(updatePhoneOrEmailReq);
    }

    @ApiOperation(value = "修改用户头像")
    @PostMapping("/updateUserAvatars")
    public R updateUserAvatars(MultipartFile file) throws Exception {

        String pattern = ".*\\.(jpg|png|gif)$";
        boolean hasExtension = Pattern.matches(pattern, file.getOriginalFilename());
        if (!hasExtension){
            String[] allowedExtension = new String[]{"jpg", "png", "gif"};
            fileUploadUtils.assertAllowed(file,allowedExtension);
        }
        Integer currentUserId = httpSessionService.getCurrentUserId();
        return userService.updateUserAvatars(currentUserId.toString(), file);
    }

}
