package com.huoran.iasf.controller;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huoran.iasf.common.annotation.Decrypt;
import com.huoran.iasf.common.aop.annotation.LogAnnotation;
import com.huoran.iasf.common.aop.annotation.NoRepeatSubmit;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysFilesEntity;
import com.huoran.iasf.entity.UserGroup;
import com.huoran.iasf.service.UserGroupService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * <p>
 * 用户组 前端控制器
 * </p>
 *
 * @author cheney
 * @since 2022-08-10
 */
@RestController
@RequestMapping("/sys/userGroup")
@Api(tags = "用户组管理")
public class UserGroupController {

    @Autowired
    private UserGroupService userGroupService;
    @NoRepeatSubmit
    @Decrypt
    @PostMapping("/add")
    @ApiOperation(value = "新增用户组")
    @LogAnnotation(title = "用户组管理", action = "新增用户组")
    @RequiresPermissions("sys:userGroup:add")
    public R addUserGroup(@RequestBody @Valid UserGroup userGroup) throws UnsupportedEncodingException {
        //中文转码
        String groupName = URLDecoder.decode(userGroup.getGroupName(), StandardCharsets.UTF_8.toString());
        userGroup.setGroupName(groupName);
        String description = URLDecoder.decode(userGroup.getDescription(), StandardCharsets.UTF_8.toString());
        userGroup.setDescription(description);
        UserGroup one = userGroupService.getOne(new QueryWrapper<UserGroup>().
                eq("group_name", userGroup.getGroupName()));
        if (ObjectUtil.isNotNull(one)){
            return R.fail("用户组已存在");
        }
        userGroupService.save(userGroup);
        return R.success();
    }
    @NoRepeatSubmit
    @Decrypt
    @PostMapping("/deleteUserGroup")
    @ApiOperation(value = "删除用户组")
    @LogAnnotation(title = "用户组管理", action = "删除用户组")
    @RequiresPermissions("sys:userGroup:deleted")
    public R deleted(@RequestBody Integer id) {
        userGroupService.removeById(id);
        return R.success();
    }

    @PostMapping("/updateUserGroup")
    @Decrypt
    @ApiOperation(value = "更新用户组信息")
    @LogAnnotation(title = "用户组管理", action = "更新用户组信息")
    @RequiresPermissions("sys:userGroup:update")
    public R updateUserGroup(@RequestBody UserGroup userGroup) throws UnsupportedEncodingException {
        if (StringUtils.isEmpty(userGroup.getId())) {
            return R.fail("id不能为空");
        }
        //中文转码
        String groupName = URLDecoder.decode(userGroup.getGroupName(), StandardCharsets.UTF_8.toString());
        userGroup.setGroupName(groupName);
        String description = URLDecoder.decode(userGroup.getDescription(), StandardCharsets.UTF_8.toString());
        userGroup.setDescription(description);
        QueryWrapper<UserGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("group_name", userGroup.getGroupName());
        queryWrapper.last(" and id  != " + userGroup.getId());
        List<UserGroup> groups = userGroupService.list(queryWrapper);
        if (ObjectUtil.isNotNull(groups) && groups.size()>0){
            return R.fail("用户组已存在");
        }
        userGroupService.updateById(userGroup);
        return R.success();
    }

/*    @GetMapping("/detail/{id}")
    @ApiOperation(value = "查询用户组详情接口")
    @LogAnnotation(title = "用户组管理", action = "查询用户组详情")
    @RequiresPermissions("sys:userGroup:detail")
    public R detailInfo(@PathVariable("id") Integer id) {
        return R.success(userGroupService.getById(id));
    }*/


    @PostMapping("/list")
    @Decrypt
    @ApiOperation(value = "用户组列表")
    @LogAnnotation(title = "用户组管理", action = "获取所有用户组机构")
    @RequiresPermissions("sys:userGroup:list")
    public R getUserGroupAll(@RequestBody UserGroup userGroup) throws UnsupportedEncodingException {
        if (!StringUtils.isEmpty(userGroup.getGroupName())) {
            //中文转码
            String groupName = URLDecoder.decode(userGroup.getGroupName(), StandardCharsets.UTF_8.toString());
            userGroup.setGroupName(groupName);
        }
        LambdaQueryWrapper<UserGroup> queryWrapper = Wrappers.lambdaQuery();
        if (!StringUtils.isEmpty(userGroup.getGroupName())) {
            queryWrapper.like(UserGroup::getGroupName, userGroup.getGroupName());
        }
        queryWrapper.orderByDesc(UserGroup::getCreateTime);
        IPage<SysFilesEntity> iPage = userGroupService.page(userGroup.getQueryPage(),queryWrapper);
        return R.success(iPage);
    }

}

