package com.huoran.iasf.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.annotation.TableField;
import lombok.Data;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotNull;

/**
 * @描述：栏目基础信息
 * @作者: Rong
 * @日期: 2022-08-03
 */
@Data
@ApiModel(value = "栏目基础信息")
@TableName("sys_column")
public class SysColumn implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId(type = IdType.AUTO)
    @ApiModelProperty(value = "id")
    private Integer id;

    @ApiModelProperty(value = "父id(第一级为0)")
    private Integer fatherId;

    @ApiModelProperty(value = "排序")
    private Integer sort;

    @ApiModelProperty(value = "层级((1-一级分类 2-二级分类 3-三级分类以此叠加))")
    private Integer level;

    @ApiModelProperty(value = "栏目名称")
    private String columnName;

    @ApiModelProperty(value = "banner路径")
    private String columnBanner;

    @ApiModelProperty(value = "分页条数")
    private Integer pageSize;

    @ApiModelProperty(value = "菜单是否可见(默认0可见 1不可见)")
    private Integer menuVisible;

    @ApiModelProperty(value = "栏目类型id")
    private Integer typeId;

    @ApiModelProperty(value = "栏目模板id")
    private Integer templateId;

    @ApiModelProperty(value = "列表样式id")
    private Integer listStyleId;

    @ApiModelProperty(value = "详情样式id")
    private Integer detailStyleId;

    @ApiModelProperty(value = "链接类型")
    private Integer connectionType;

    @ApiModelProperty(value = "链接地址")
    private String linkAddress;

    @ApiModelProperty(value = "只有一篇文章时，以详情方式展示(0默认否 1为是)")
    private Integer showWithDetails;

    @ApiModelProperty(value = "站点选择(连接类型为其它站点连接时储存)1.EFL中文2.SRL中文")
    private Integer siteSelection;

    @ApiModelProperty(value = "创建人id")
    @NotNull(message = "创建人id不能为空")
    private Integer founderId;

    @ApiModelProperty(value = "编辑人id")
    @NotNull(message = "编辑人id能为空")
    private Integer editorId;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty(value = "状态(0默认草稿箱 1为已发布)")
    private Integer status;

    @ApiModelProperty(value = "是否删除(1未删除；0已删除)")
    @TableLogic
    private Integer deleted;


    @ApiModelProperty(value = "是否在新窗口打开(0:否 1:是)")
    private Integer isOpen;

    @ApiModelProperty(value = "站点id")
    @NotNull(message = "站点id不能为空!")
    private Integer siteId;


    @ApiModelProperty(value = "栏目副标题")
    private String subtitle;


    //----------------------非数据库字段Start----------------------

    @ApiModelProperty("子分类")
    @TableField(exist = false)
    private List<SysColumn> children;


    @TableField(exist = false)
    @ApiModelProperty(value = "创建人")
    private String creteUser;


    @TableField(exist = false)
    @ApiModelProperty(value = "修改人")
    private String editorUser;


    @TableField(exist = false)
    @ApiModelProperty(value = "总浏览")
    private Integer totalBrowsing;

    public Integer getTotalBrowsing() {
        return 0;
    }

    @TableField(exist = false)
    @ApiModelProperty(value = "模板名称")
    private String templateName;

    @TableField(exist = false)
    @ApiModelProperty(value = "列表样式")
    private String listStyle;


    @TableField(exist = false)
    @ApiModelProperty(value = "详情样式")
    private String detailStyle;

    @TableField(exist = false)
    @ApiModelProperty(value = "路径")
    private String path;

    //----------------------非数据库字段End----------------------

}