package com.huoran.iasf.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.annotation.TableField;
import lombok.Data;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotNull;

/**
 * @描述：文章管理
 * @作者: Rong
 * @日期: 2022-08-08
 */
@Data
@ApiModel(value = "文章管理")
@TableName("sys_content")
public class SysContent implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId(type = IdType.AUTO)
    @ApiModelProperty(value = "主键")
    private Integer id;

    @ApiModelProperty(value = "标题(全库判重)")
    private String title;

    @ApiModelProperty(value = "所属栏目")
    private Integer columnId;

    @ApiModelProperty(value = "发布时间")
    private String releaseTime;

    @ApiModelProperty(value = "来源")
    private String source;

    @ApiModelProperty(value = "作者")
    private String author;

    @ApiModelProperty(value = "banner图")
    private String bannerImg;

    @ApiModelProperty(value = "文章模板(前端自定义)")
    private Integer articleTemplate;

    @ApiModelProperty(value = "摘要")
    private String summary;

    @ApiModelProperty(value = "标题图(封面图)")
    private String titleImg;

    @ApiModelProperty(value = "正文")
    private String mainBody;

    @ApiModelProperty(value = "是否发布(0:草稿 1:已发布)")
    private Integer isRelease;

    @ApiModelProperty(value = "创建人id")
    private Integer founderId;

    @ApiModelProperty(value = "编辑人id")
    private Integer editorId;

    @ApiModelProperty(value = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;
    @ApiModelProperty(value = "文章总浏览量")
    private Integer totalBrowsing;

    @ApiModelProperty(value = "是否在新窗口打开(0:否 1:是)")
    private Integer isOpen;

    @ApiModelProperty(value = "是否删除(1未删除；0已删除)")
    @TableLogic
    private Integer deleted;

    @ApiModelProperty(value = "站点id")
    private Integer siteId;

    @ApiModelProperty(value = "模板状态(0禁用；1启用)")
    private Integer templateStatus;

   /* @ApiModelProperty(value = "其余模板字段")
    private String remainingTemplateFields;*/

    @ApiModelProperty(value = "链接类型")
    private Integer connectionType;

    @ApiModelProperty(value = "链接地址")
    private String linkAddress;

    @ApiModelProperty(value = "站点选择(连接类型为其它站点连接时储存)")
    private Integer siteSelection;

    @ApiModelProperty(value = "编辑人")
    private String edit;

    @ApiModelProperty(value = "审核人")
    private String audit;

    @ApiModelProperty(value = "活动开始时间")
    private String activityStartTime;

    @ApiModelProperty(value = "活动结束时间")
    private String activityEndTime;

    @ApiModelProperty(value = "演讲系列")
    private String lectureSeries;

    @ApiModelProperty(value = "线上地点")
    private String onlineLocation;

    @ApiModelProperty(value = "线下地点")
    private String offlineLocation;

    @ApiModelProperty(value = "主讲人")
    private String keynoteSpeaker;

    @ApiModelProperty(value = "活动简介")
    private String eventProfile;

    @ApiModelProperty(value = "出版年份")
    private String publicationYear;

    @ApiModelProperty(value = "出版类型(前端自定义)")
    private Integer publicationTypeId;

    @ApiModelProperty(value = "DOI")
    private String doi;

    @ApiModelProperty(value = "引用")
    private String quote;

    @ApiModelProperty(value = "主题标签(sys_content_label表外键) 可多选逗号隔开")
    private String lableId;

    @ApiModelProperty(value = "所属分类(sys_content_classification表外键)")
    private Integer classificationId;

    @ApiModelProperty(value = "是否禁用(0默认，0启用 1禁用)")
    private Integer isDisable;

    @ApiModelProperty(value = "栏目名称")
    @TableField(exist = false)
    private String columnName;

    @ApiModelProperty(value = "文章附件")
    @TableField(exist = false)
    List<SysContentFile> fileList;


    @ApiModelProperty(value = "刊物名称")
    private String periodicalName;

    @ApiModelProperty(value = "卷")
    private String reel;

    @ApiModelProperty(value = "文献号")
    private String documentNumber;

    @ApiModelProperty(value = "文章关键字")
    private String articleKeyWord;

    /*@ApiModelProperty(value = "顺序(排序号)")
    private Integer sequence;*/

    @ApiModelProperty(value = "是否置顶(默认为0 不置顶 1为置顶)")
    private Integer isTop;

    @ApiModelProperty(value = "置顶时间")
    private Date topTime;
    @ApiModelProperty(value = "专利类别id")
    private Integer patentClassId;

    @ApiModelProperty(value = "申请时间")
    private String applicationDate;

    @ApiModelProperty(value = "授权日期")
    private String dateOfAuthorization;

    @ApiModelProperty(value = "发明人")
    private String inventor;

    @ApiModelProperty(value = "专利名称")
    private String patentName;

    @ApiModelProperty(value = "申请号")
    private String applicationNumber;

    @ApiModelProperty(value = "出版社")
    private String publishingHouse;

    @ApiModelProperty(value = "著名著作")
    private String famousWorks;

    @ApiModelProperty(value = "编写人员")
    private String writersAndEditors;

    @ApiModelProperty(value = "出版日期")
    private String publicationTime;


    @ApiModelProperty(value = "岗位")
    private String post;

    @ApiModelProperty(value = "专业")
    private String major;
    @ApiModelProperty(value = "爱好")
    private String hobby;
    @ApiModelProperty(value = "荣誉")
    private String honor;
}