package com.huoran.iasf.service;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * session管理器
 * @author cheney
 * @version V1.0
 * @date 2022年5月11日
 */
@Service
public class HttpApiSessionService {


    public static final String USER_ID_KEY = "user_id_key";

    public static final String USER_USERNAME_KEY = "user_username_key";

    public static final String SUBJECT = "HuoRan_SUBJECT";

    public static final long EXPIRE = 1000 * 60 * 60 * 24;  //过期时间，毫秒，一天

    //秘钥
    public static final String APP_SECRET = "HuoRan_SYMMETRIC_ENCRYPTION_HS256";

    @Resource
    private HttpServletRequest request;

    /**
     * 生成jwt
     * @param userId username
     * @return token
     */
    public String generateToken(Integer userId, String userName) {
        String token = Jwts.builder().setSubject(SUBJECT)
                .claim(USER_ID_KEY, userId)
                .claim(USER_USERNAME_KEY, userName)
                .setIssuedAt(new Date())
                .setExpiration(new Date(System.currentTimeMillis() + EXPIRE))
                .signWith(SignatureAlgorithm.HS256, APP_SECRET).compact();
        return token;
    }


    /**
     * 校验token
     *
     * @param token
     * @return
     */
    public Claims checkJWT(String token) {

        try {
            final Claims claims = Jwts.parser().setSigningKey(APP_SECRET).
                    parseClaimsJws(token).getBody();
            return claims;
        } catch (Exception e) {
        }
        return null;

    }

    /**
     * 获取当前session信息 username
     *
     * @return username
     */
    public String getCurrentUsername() {
        if (request.getAttribute(USER_USERNAME_KEY) != null) {
            return request.getAttribute(USER_USERNAME_KEY).toString();
        } else {
            return null;
        }
    }

    /**
     * 获取当前session信息 UserId
     *
     * @return UserId
     */
    public String getCurrentUserId() {
        if (request.getAttribute(USER_ID_KEY) != null) {
            return request.getAttribute(USER_ID_KEY).toString();
        } else {
            return null;
        }
    }


}
