package com.huoran.iasf.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.huoran.iasf.entity.SysPermission;
import com.huoran.iasf.vo.resp.PermissionRespNode;

import java.util.List;
import java.util.Set;

/**
 * 菜单权限
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
public interface PermissionService extends IService<SysPermission> {

    /**
     * 根据userId获取权限
     *
     * @param userId userId
     * @return 权限
     */
    List<SysPermission> getPermission(Integer userId);

    /**
     * 删除权限
     *
     * @param permissionId 权限id
     */
    void deleted(Integer permissionId);

    /**
     * 获取所有
     *
     * @return List
     */
    List<SysPermission> selectAll();

    /**
     * 根据userId获取权限标志
     *
     * @param userId userId
     * @return Set
     */
    Set<String> getPermissionsByUserId(Integer userId);

    /**
     * 根据userId获取权限树
     *
     * @param userId
     * @return List
     */
    List<PermissionRespNode> permissionTreeList(Integer userId);

    /**
     * 根据权限树
     *
     * @return List
     */
    List<SysPermission> selectAllByTree();

    /**
     * 根据目录树
     *
     * @param permissionId permissionId
     * @return List
     */
    List<PermissionRespNode> selectAllMenuByTree(Integer permissionId);


    /**
     * 根据权限id获取绑定的userId
     *
     * @param permissionId permissionId
     * @return List
     */
    List<Integer> getUserIdsById(Integer permissionId);

    /**
     * 更新
     *
     * @param vo vo
     */
    void updatePermission(SysPermission vo);

    List<SysPermission> permissionList(Integer id);
}
