package com.huoran.iasf.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.iasf.common.exception.BusinessException;
import com.huoran.iasf.common.exception.code.BaseResponseCode;
import com.huoran.iasf.entity.SysRole;
import com.huoran.iasf.entity.SysRolePermission;
import com.huoran.iasf.entity.SysUserRole;
import com.huoran.iasf.mapper.SysRoleMapper;
import com.huoran.iasf.service.HttpSessionService;
import com.huoran.iasf.service.RolePermissionService;
import com.huoran.iasf.service.RoleService;
import com.huoran.iasf.service.UserRoleService;
import com.huoran.iasf.vo.req.RolePermissionOperationReqVO;
import com.huoran.iasf.vo.resp.DeptRespNodeVO;
import com.huoran.iasf.vo.resp.PermissionRespNode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 角色
 *
 * @author cheney
 * @version V1.0
 * @date 2022年7月28日
 */
@Service
@Slf4j
public class RoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements RoleService {
    @Resource
    private SysRoleMapper sysRoleMapper;
    @Resource
    private UserRoleService userRoleService;
    @Resource
    private RolePermissionService rolePermissionService;
    @Resource
    private HttpSessionService httpSessionService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addRole(SysRole vo) {

        vo.setStatus(1);
        sysRoleMapper.insert(vo);
        if (!CollectionUtils.isEmpty(vo.getPermissions())) {
            RolePermissionOperationReqVO reqVO = new RolePermissionOperationReqVO();
            reqVO.setRoleId(vo.getId());
            reqVO.setPermissionIds(vo.getPermissions());
            rolePermissionService.addRolePermission(reqVO);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateRole(SysRole vo) {
        SysRole sysRole = sysRoleMapper.selectById(vo.getId());
        if (null == sysRole) {
            log.error("传入 的 id:{}不合法", vo.getId());
            throw new BusinessException(BaseResponseCode.DATA_ERROR);
        }
        sysRoleMapper.updateById(vo);
        //删除角色权限关联
        rolePermissionService.remove(Wrappers.<SysRolePermission>lambdaQuery().eq(SysRolePermission::getRoleId, sysRole.getId()));
        if (!CollectionUtils.isEmpty(vo.getPermissions())) {
            RolePermissionOperationReqVO reqVO = new RolePermissionOperationReqVO();
            reqVO.setRoleId(sysRole.getId());
            reqVO.setPermissionIds(vo.getPermissions());
            rolePermissionService.addRolePermission(reqVO);
            // 刷新权限
            httpSessionService.refreshRolePermission(sysRole.getId());
        }
    }


    private void setCheckedDept(List<DeptRespNodeVO> deptRespNodes, Set<Object> checkDeptList) {
        for (DeptRespNodeVO node : deptRespNodes) {
            if (checkDeptList.contains(node.getId())) {
                // node.setChecked(true);
            }
            setCheckedDept((List<DeptRespNodeVO>) node.getChildren(), checkDeptList);
        }
    }


    private void setChecked(List<PermissionRespNode> list, Set<Object> checkList) {
        for (PermissionRespNode node : list) {
            if (checkList.contains(node.getId())
                    && CollectionUtils.isEmpty(node.getChildren())) {
                // node.setChecked(true);
            }
            setChecked((List<PermissionRespNode>) node.getChildren(), checkList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void deletedRole(List<String> ids) {

        for (String id : ids) {
            //获取关联userId
            List<Integer> userIds = userRoleService.getUserIdsByRoleId(Integer.valueOf(id));
            //删除角色
            sysRoleMapper.deleteById(id);
            //删除角色权限关联
            rolePermissionService.remove(Wrappers.<SysRolePermission>lambdaQuery().eq(SysRolePermission::getRoleId, id));
            //删除角色用户关联
            userRoleService.remove(Wrappers.<SysUserRole>lambdaQuery().eq(SysUserRole::getRoleId, id));
            if (!CollectionUtils.isEmpty(userIds)) {
                // 刷新权限
                userIds.parallelStream().forEach(httpSessionService::refreshUerId);
            }
        }
    }

    @Override
    public List<SysRole> getRoleInfoByUserId(Integer userId) {

        List<Integer> roleIds = userRoleService.getRoleIdsByUserId(userId);
        if (CollectionUtils.isEmpty(roleIds)) {
            return null;
        }
        return sysRoleMapper.selectBatchIds(roleIds);
    }

    @Override
    public List<String> getRoleNames(Integer userId) {
        List<SysRole> sysRoles = getRoleInfoByUserId(userId);
        if (CollectionUtils.isEmpty(sysRoles)) {
            return null;
        }
        return sysRoles.stream().map(SysRole::getName).collect(Collectors.toList());
    }

    @Override
    public boolean checkDeptRepeat(SysRole vo) {
        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", vo.getName());
        if (vo.getId()!=null) {
            queryWrapper.last(" and id  != " + vo.getId());
        }
        List<SysRole> sysRoles = baseMapper.selectList(queryWrapper);
        return sysRoles.size() > 0;
    }
}
