package com.huoran.iasf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.common.utils.TreeStructureUtils;
import com.huoran.iasf.entity.SysColumn;
import com.huoran.iasf.mapper.SysColumnMapper;
import com.huoran.iasf.mapper.SysContentMapper;
import com.huoran.iasf.service.SysColumnService;
import com.huoran.iasf.vo.FatherContentRespVO;
import com.huoran.iasf.vo.req.ColumnWeightReqVO;
import com.huoran.iasf.vo.req.LongPageColumnReqVO;
import com.huoran.iasf.vo.req.PageContentReqVO;
import com.huoran.iasf.vo.req.PaginationColumnReqVO;
import com.huoran.iasf.vo.resp.PageContentRespVO;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @描述： 服务类
 * @作者: Rong
 * @日期: 2022-08-01
 */
@Service
public class SysColumnServiceImpl extends ServiceImpl<SysColumnMapper, SysColumn> implements SysColumnService {

    @Autowired
    private SysColumnMapper mapper;

    @Autowired
    private SysContentMapper contentMapper;

    @Override
    public boolean sameLevelJudgment(ColumnWeightReqVO column) {
        QueryWrapper<SysColumn> queryWrapper = new QueryWrapper<>();


        queryWrapper.eq("column_name", column.getColumnName());

        if (column.getId() != null) {
            queryWrapper.eq("father_id", column.getId());
        } else {
            queryWrapper.eq("father_id", 0);
        }


        queryWrapper.eq("site_id", column.getSiteId());
        queryWrapper.eq("deleted", 1);
        SysColumn sysColumn = mapper.selectOne(queryWrapper);
        if (sysColumn != null) {
            if (column.getId() != null) {
                //判断为编辑情况下，如果编辑的id与查询出来的一致不提示
                if (column.getId() == sysColumn.getId()) {
                    return true;
                }
            }

            return false;
        }
        return true;
    }

    @Override
    public List<SysColumn> listWithTree(PaginationColumnReqVO column) {
        //查询所有栏目
        List<SysColumn> columnList = baseMapper.filter(column);

        if (column.getColumnName().equals("")) {
            //用来保存父节点
            List<SysColumn> rootColumn = new ArrayList<>();
            //用来保存子节点
            List<SysColumn> childList = new ArrayList<>();
            //遍历所有栏目，将所有栏目的父栏目id为0的栏目放入父节点集合中
            for (SysColumn column1 : columnList) {
                Integer parentId = column1.getFatherId();
                if (parentId == 0) {
                    rootColumn.add(column1);
                }
            }
            for (SysColumn sysColumn : rootColumn) {
                Integer theColumnId = sysColumn.getId();
                //调用工具类获取子节点
                childList = TreeStructureUtils.getChildren(theColumnId, columnList);
                sysColumn.setChildren(childList);
            }

            return rootColumn;
        }

        return columnList;


    }

    @Override
    public List<SysColumn> listWithTreeMenuVisible(PaginationColumnReqVO column) {

        if (column.getRole() == null) {

        } else if (column.getRole() != null || column.getRole().equals("admin") || ObjectUtil.isNotNull(column.isIsadmin()) || column.isIsadmin()) {
            throw new IllegalArgumentException("参数名不能包含admin");
        }
        //查询所有栏目
        List<SysColumn> columnList = baseMapper.filterMenuVisible(column);

        if (column.getColumnName().equals("")) {
            //用来保存父节点
            List<SysColumn> rootColumn = new ArrayList<>();
            //用来保存子节点
            List<SysColumn> childList = new ArrayList<>();
            //遍历所有栏目，将所有栏目的父栏目id为0的栏目放入父节点集合中
            for (SysColumn column1 : columnList) {
                Integer parentId = column1.getFatherId();
                if (parentId == 0) {
                    rootColumn.add(column1);
                }
            }
            for (SysColumn sysColumn : rootColumn) {
                Integer theColumnId = sysColumn.getId();
                //调用工具类获取子节点
                childList = TreeStructureUtils.getChildren(theColumnId, columnList);
                sysColumn.setChildren(childList);
            }

            return rootColumn;
        }

        return columnList;

    }

    @Override
    public R longPageColumnList(LongPageColumnReqVO reqVO) {
        Page<PageContentRespVO> page = new Page<PageContentRespVO>(reqVO.getPageNum(), reqVO.getPageSize());
        IPage<PageContentRespVO> pageList = baseMapper.longPageColumnList(page, reqVO);
        pageList.getRecords().forEach(pageContentRespVO -> {
            if (pageContentRespVO.getLevel() > 1) {
                //判断该栏目层级为第一级时候不返回上级栏目
                List<FatherContentRespVO> upperLevel = baseMapper.getParentInformationBasedOnChild(pageContentRespVO.getId());
                pageContentRespVO.setSuperiorColumn(upperLevel);
            }

        });


        return R.success(pageList);
    }

    @Override
    @CacheEvict(value = "articles", key = "#id")
    public void removeArticles(Integer id) {

    }

    @Override
    public R getsTheSubColumn(Integer pid, String ids,Integer siteId) {

        List<SysColumn> getsTheSubColumn = baseMapper.getsTheSubColumn(pid, ids, null,siteId);
        return R.success(TreeStructureUtils.buildTree(getsTheSubColumn, pid));
    }

    @Override
    public R getsSublevelColumnsUnderALevel(Integer parentId, String ids,Integer siteId) {

        List<SysColumn> getsTheSubColumn = baseMapper.getsTheSubColumn(parentId, ids, null,siteId);
        List<SysColumn> buildTree = TreeStructureUtils.buildTree(getsTheSubColumn, parentId);
        return R.success(buildTree);
    }

    @Override
    public Integer oneLevelChecksThemAll(Integer theChildId) {
        Integer getPid = baseMapper.oneLevelChecksThemAll(theChildId);
        return getPid;
    }

    @Override
    public R getSubColumn(Integer parentId, String ids, Integer isSort,Integer siteId) {
        List<SysColumn> getsTheSubColumn = baseMapper.getsTheSubColumn(parentId, ids, isSort,siteId);
        return R.success(TreeStructureUtils.forMethod(getsTheSubColumn));
    }

    @Override
    @Cacheable(value = "articles", key = "#id")
    public R queryArticlesByColumnType(String ids, Integer id) {

        Integer count = 0;
        for (String columnId : ids.split(",")) {
            ++count;
        }


        List<SysColumn> queryByColumnType = baseMapper.queryByColumnType(ids);
        //为1表示该层级下以及它本身列表样式模板一直则查询全部的
        if (count > 1) {
            if (queryByColumnType.size() == 1) {
                return R.success(contentMapper.columnConditionsFilterArticles(ids));
            }
        }
        //表示该层级下的全部层级不一致只查本身
        return R.success(contentMapper.columnConditionsFilterArticles(id + ""));
    }


}



