package com.huoran.iasf.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huoran.iasf.common.utils.Constant;
import com.huoran.iasf.common.utils.R;
import com.huoran.iasf.entity.SysContent;
import com.huoran.iasf.entity.SysContentFile;
import com.huoran.iasf.mapper.SysContentFileMapper;
import com.huoran.iasf.mapper.SysContentMapper;
import com.huoran.iasf.service.SysContentService;
import com.huoran.iasf.vo.req.ArticleModifiedSortReq;
import com.huoran.iasf.vo.req.ContentHeavyTitleReqVO;
import com.huoran.iasf.vo.req.PageContentReqVO;
import com.huoran.iasf.vo.resp.PageContentRespVO;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @描述：文章管理 服务类
 * @作者: Rong
 * @日期: 2022-08-05
 */
@Service
public class SysContentServiceImpl extends ServiceImpl<SysContentMapper, SysContent> implements SysContentService {

    private final static Logger logger = LoggerFactory.getLogger(SysContentServiceImpl.class);
    @Autowired
    private SysContentMapper mapper;

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private SysContentFileMapper fileMapper;

    @Override
    public boolean checkIfTheTitleIsRepeat(ContentHeavyTitleReqVO content) {
        //只对已发布的标题判重
        QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("title", content.getTitle());
        queryWrapper.eq("is_release", Constant.ARTICLE_PUBLISHED);
        queryWrapper.eq("site_id", content.getSiteId());
        SysContent sysContent = mapper.selectOne(queryWrapper);
        if (sysContent != null) {
            if (content.getId() != null) {
                //判断为编辑情况下，如果编辑的id与查询出来的一致不提示
                if (content.getId().equals(content.getId())) {
                    return true;
                }
            }

            return false;
        }
        return true;
    }

    @Override
    public R articlePaginationList(PageContentReqVO reqVO) {
        Page<PageContentRespVO> page = new Page<PageContentRespVO>(reqVO.getPageNum(), reqVO.getPageSize());
        IPage<PageContentRespVO> pageList = baseMapper.articlePaginationList(page, reqVO);
        return R.success(pageList);
    }

    @Override
    public R newlyPublishedArticles(PageContentReqVO reqVO) {
        if (reqVO.getRole() == null) {

        } else if (reqVO.getRole() != null || reqVO.getRole().equals("admin") || ObjectUtil.isNotNull(reqVO.isIsadmin()) || reqVO.isIsadmin()) {
            throw new IllegalArgumentException("参数名不能包含admin");
        }
        Page<PageContentRespVO> page = new Page<PageContentRespVO>(reqVO.getPageNum(), reqVO.getPageSize());
        IPage<PageContentRespVO> pageList = baseMapper.getPublishedArticles(page, reqVO);


        for (PageContentRespVO vo : pageList.getRecords()) {
            List<SysContentFile> fileList = fileMapper.getFileByContentId(vo.getId());
            vo.setFileList(fileList);
        }
        return R.success(pageList);
    }

    @Override
    public R publishedEnabledArticles() {
        //查询已发布的、启用了的文章列表接
        QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_release", Constant.ARTICLE_PUBLISHED);
        queryWrapper.eq("template_status", Constant.TEMPLATE_STATUS_ENABLED);
        List<SysContent> sysContentList = mapper.selectList(queryWrapper);
        return R.success(sysContentList);

    }

    @Override
    public R getTheFullArticleByColumn(String ids) {
        return R.success(mapper.useTheColumnToGetTagsForTheFullArticle(ids));
    }

    @Override
    public Integer statisticsOfPageViews(Integer id) {

        String key = "preview:contentId:" + id;
        //查看缓存是否有数据
        String returnValue = redisTemplate.opsForValue().get(key);
        if (returnValue == null) {
            returnValue = "0";
        }
        returnValue = Integer.valueOf(returnValue) + 1 + "";
        //保存到缓存中
        redisTemplate.opsForValue().set(key, returnValue + "");

        SysContent sysContent = new SysContent();
        sysContent.setId(id);
        sysContent.setTotalBrowsing(Integer.valueOf(returnValue));
        mapper.updateById(sysContent);

        return Integer.valueOf(returnValue);
    }

    @Override
    public List<SysContent> hotContent(Integer siteId) {
        QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("deleted", Constant.DATA_NOT_DELETED);
        queryWrapper.eq("site_id", siteId);
        queryWrapper.eq("is_release", Constant.ARTICLE_PUBLISHED);
        queryWrapper.eq("is_disable", 0);
        queryWrapper.last(" ORDER BY total_browsing desc LIMIT 0,5");

        List<SysContent> contentList = baseMapper.selectList(queryWrapper);
        return contentList;
    }

    @Override
    public R siteSearchArticles(PageContentReqVO reqVO) {
        Page<SysContent> page = new Page<SysContent>(reqVO.getPageNum(), reqVO.getPageSize());
        IPage<SysContent> pageList = baseMapper.siteSearchArticles(page, reqVO);
        return R.success(pageList);
    }

    @Override
    public R queryArticleColumnParent(Integer articleId) {
        SysContent sysContent = baseMapper.selectById(articleId);

        return R.success(baseMapper.queryArticleColumnParent(sysContent.getColumnId()));
    }

    /*@Override
    public R modifiedSort(List<ArticleModifiedSortReq> reqList) {
        for (ArticleModifiedSortReq req : reqList) {
            SysContent content = new SysContent();
            content.setId(req.getArticleId());
            content.setSequence(req.getSequence());
            baseMapper.updateById(content);
        }
        return R.success();
    }

    @Override
    public R modifiedSort(Integer sequenceNumber, Integer articleId) {

        // 占分布式锁，去redis占坑
        // 1. 分布式锁占坑
        Boolean lock = redisTemplate.opsForValue().setIfAbsent("articleSortLock", "value", 30, TimeUnit.SECONDS);
        if (lock) {
            //加锁成功...
            if (sequenceNumber <= 0) {
                redisTemplate.delete("articleSortLock");   //删除key，释放锁
                return R.fail("请输入正确的序号!");
            }
            SysContent content = baseMapper.selectById(articleId);
            //获取当前排序的序号
            Integer currentSerialNumber = content.getSequence();
            //获取当前文章所属栏目id
            Integer columnId = content.getColumnId();

            if (currentSerialNumber.equals(sequenceNumber)) {
                redisTemplate.delete("articleSortLock");   //删除key，释放锁
                return R.success();
            }
            //修改当前序号
            UpdateWrapper<SysContent> updateWrapper = new UpdateWrapper<>();
            updateWrapper.set("sequence", sequenceNumber);
            updateWrapper.eq("id", articleId);
            //做乐观锁处理
            updateWrapper.eq("sequence", currentSerialNumber);
            int updateCount = baseMapper.update(new SysContent(), updateWrapper);

            UpdateWrapper<SysContent> updateWrapper1 = new UpdateWrapper<>();
            updateWrapper1.ne("id", articleId);
            //currentSerialNumber 当前文章的序号
            //sequenceNumber 要更改的序号

            *//**
     * 就要判断排序是值改小还是改大
     * 大改小比如53改5那就加一个语句“大于等于5并且小于53”
     * 如果是小改大比如3改50那就是“大于3并且小于等于50"
     *//*
            if (currentSerialNumber > sequenceNumber) {
                //改大 gt >
                updateWrapper1.setSql("  sequence = sequence + 1");
                updateWrapper1.last(" and sequence >= " + sequenceNumber + " and sequence < " + currentSerialNumber);
            } else if (currentSerialNumber < sequenceNumber) {
                //改小
                //le为<=
//                updateWrapper1.le("sequence", sequenceNumber);
                updateWrapper1.setSql("  sequence = sequence - 1");
                updateWrapper1.last(" and sequence > " + currentSerialNumber + " and sequence <= " + sequenceNumber);
                //用于判断序号为本身就为1时候就不减了
//                updateWrapper1.ne("sequence", 1);
            }

            updateWrapper1.eq("column_id", columnId);
            updateWrapper1.eq("deleted", 1);
            baseMapper.update(new SysContent(), updateWrapper1);
            redisTemplate.delete("articleSortLock");   //删除key，释放锁
            return R.success();
        } else {
            return R.fail("当前有用户操作排序，请稍后刷新重试!");
        }


    }*/

    /*@Override
    public R modifiedSort(Integer newIndex, Integer articleId,Integer sort) {
        if (newIndex <= 0) {
            return R.fail("请输入正确的序号!");
        }
        //获取当前排序的序号
        SysContent content = baseMapper.selectById(articleId);
        //获取当前文章所属栏目id
        Integer columnId = content.getColumnId();
        //查看当前升序情况下全部文章信息
        QueryWrapper<SysContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("column_id", columnId);
        queryWrapper.eq("deleted", 1);

        if (sort==0){
            queryWrapper.orderByAsc("sequence");
        }else if (sort==1){
            queryWrapper.orderByDesc("sequence");
        }

        List<SysContent> contentList = baseMapper.selectList(queryWrapper);
        //获取当前序号在list中的顺序
        int currentIndex = contentList.indexOf(content);
        //获取当前排序的序号
        Integer currentSerialNumber = content.getSequence();
        if (currentIndex == -1) {
            // 文章不存在，不做任何操作
            return R.success();
        }
        //对于传过来的序号 处理用于匹配集合的下标
        if (currentSerialNumber > newIndex) {
            newIndex--;
        }else {
            newIndex++;
        }

        if (currentIndex == newIndex) {
            // 新旧位置相同，不做任何操作
            return R.success();
        }
        // 移除当前位置的文章
        contentList.remove(content);
        if (newIndex >= contentList.size()) {
            // 插入到末尾
            contentList.add(content);
        } else {
            // 插入到新位置
            contentList.add(newIndex, content);
        }
        for (int i = 0; i < contentList.size(); i++) {
            SysContent article = contentList.get(i);
            article.setSequence(i + 1);
            baseMapper.updateById(article);
        }


        return R.success();
    }*/


}




